C     GRINSP-P - Version parallelized with OpenMP - October 2006
C
C     Geometrically Restrained INorganic Structure Prediction
C
C           Generates positions with distance constraints
C                  in a box so as to build 3, 4, 5 or 
C                  6-connected 3D networks (T2X3, TX2 - zeolites
C                  or TX3, etc) or even mixed networks TxT'yXz.
C
C         - October-December 2003
C
C    Version 0.90 : October 2003
C            TX2 only, X close to the T-T midpoints
C            P1 space group only
C
C    Version 0.99 : December 2003
C            2 T/T' atom types and X close to the T-T midpoints
C            P1 space group only
C
C
C    Version 1.00 : March-April 2004
C            Any standard space group 
C
C    Version 2.00 : March-April 2006
C            Several bugs corrected
C            Some changes in the .dat file and in the
C               output CIF file. No more production of the .xtl files 
C            Space groups are recognized by their number
C            A series of space groups can be examined, even all space 
C            groups from 1 to 230, needing a parallelized version working
C            on a multi-processors machine...
C
C   Armel Le Bail  alb@cristal.org
C
C=====================================================================
C
C   Some definitions :  
C
C   x(j,i,1-3)  = reduced atomic coordinates of atom j of type i
C
C   d0(i,j)   = min first distance between atoms of types i and j 
C   e0(i,j)   = max first distance between atoms of types i and j 
C   f0(i,j)   = min second distance between atoms of type i and j
C   g0(i,j)   = max second distance between atoms of type i and j
C
C   nv(j,i,k) = characterizes environment k of atom j of type T
C               when k=1,4 (or 6) first neighbours T atoms around every T 
C               then the model is OK                             
C   mv(j,i)  = number of first T neighbours for atom j of type T
C                  
C              j=1,nl(1)     
C              if i=1 : T-T first neighbours are considered
C              if i=2 : T-T second neighbours are considered 
C
C  nl(1)     = number of atoms of type T already placed
C  nl(2)     = number of atoms of type X already placed
C
C=====================================================================
C
      program grinsp
C
C   nmax = max number of different atom types (2)
C   maxa = max number of atoms in one type in 27 cells
C          192*27=5184
C   maxb = max number of atoms in one type in 27x27 cells
C          192*27*27=139968
C   maxc = max coordination (6) (first neighbours 1-2)
C   ngood = max number of good solutions retained (2000)
C   nze = max number of known coordination sequences (5000)
C   nat = max number of cations (192)
C   node = max number of different cationic nodes (20)
C         (sites with different coordination sequences)
C
      parameter (maxa=5184,maxb=139968,nmax=2,maxc=6,
     1ngood=2000,nze=5000,nat=192,node=20)
      real  x(maxa,nmax,3)
      real  d0(nmax,nmax,3),e0(nmax,nmax,3),di0(nmax,nmax,3)
      real  f(2,3),g(2,3),f0(2,3),g0(2,3),f3(3),g3(3),e022(3)
      real  metn(3,3),met(3,3),rdp(ngood),rdzeot(nze)
      real  rd(ngood),rho(ngood),amem(ngood),bmem(ngood),cmem(ngood)
      real  alpmem(ngood),betmem(ngood),gammem(ngood)
      real  vmem(ngood),al(3,3),celpre(6),celpren(6),cop(2)
      real  ts(30,192,3),tt(30,192,3),sm(30,192,3,3),smt(30,192,3,3)
      real  xs(maxa),ys(maxa),zs(maxa)
      real  time_begin,time_end,totaltime
      integer  sgzeot(nze),isize
      integer  nt(ngood),nmem(ngood),nmcmem(ngood),ll(ngood)
      integer  nl(nmax),nv(maxa,nmax,maxc),nvt(maxa,maxc),mcop(maxa)
      integer  mv(maxa,nmax),ns(node),mjmax(ngood),init0(ngood)
      integer  ifname1,ncop(2),nr(node,10),mnr(ngood,node,10)
      integer  mntype(ngood),mns(ngood,node),nzeot(nze),nmzeot(nze)
      integer igr(36),igrw(36),igr2(7),igr3(7),ncs(maxa,10)
      integer ibra(7),isys(7),irl(192),nag(192),nas(192),mwyc(maxa)
      integer it1(maxa),it2(maxa),ist(maxa),nass(192),sgmem(ngood)
      character*11  zeot(nze),ident(ngood),identn(ngood),ide(ngood)
      character*40  coseq(node,nze),newcos(node,ngood)
      character*4   t(10),version,atyp(2,2),btyp(2,2)
      character*4   satyp(2,2),namt
      character*15  formula,formem(ngood)
      character*3   idt
      character*20  tchem
      character*1   actyp,bctyp,wy(192),wys(192)
      character*20  file,nam
      character*24  tempo,spgr
      character*7   fname,datenow,namemem(ngood)
      character*80  title
      CHARACTER*8 TIMENOW
      logical PRESSEDK / .FALSE. /
      logical qex
      common/dist/ y(maxb,3),ny(maxb)
      common/xy/ xy(maxa,3,maxc),nxy(maxa,maxc)
C$OMP THREADPRIVATE(/dist/,/xy/) 
C
      data igr2/'A','B','C','D','M','N','P'/
      data igr3/'a','b','c','d','m','n','p'/
      data isys/7,1,2,4,6,5,3/
      data ibra/'P','A','B','C','F','I','R'/
C
C
      PI2=6.2831853
	version='2.0P'
C
C
      CALL KMP_GET_STACKSIZE(ISIZE)
C      print *,ISIZE
      ISIZE=200000000
      CALL KMP_SET_STACKSIZE(ISIZE)
      CALL KMP_GET_STACKSIZE(ISIZE)
C      print *,ISIZE
C
C
C Open files using name from command line and standard extensions.
C The OPEN statements may need to be changed for some computers.
C Subroutine SXNM gets the generic filename from the command line.
C If nothing is found, then the user is prompted for the filename.
C
      CALL MCMNAM(LN,NAM)
C	PRINT *,LN,NAM
      IF(NAM(LN-3:LN).EQ.'.exe')GOTO 334
      FILE=NAM(1:LN)
      LFILE=LN
      GO TO 335
20    format(a20)
21    format(a80)
334   print *,'Input file (no extension) '
      read 20, file
      lfile=len(file)
      do while (file(lfile:lfile).eq.' ')
         lfile=lfile-1
      enddo
335   tempo=file(1:lfile)//'.dat'
      call open_read1(4,tempo)
      tempo=file(1:lfile)//'.imp'
      inquire(file=tempo,exist=qex)
      if(qex.eq..FALSE.) go to 201
      call filedel(3,tempo)
201   continue
      call open_write1(10,tempo)
      WRITE (10,3000) VERSION,FILE(1:LFILE)
3000  FORMAT(/1x,'=============================================
     -===================='/
     -        1x,'GRINSP version ',a,' by A. Le Bail -  2006  -
     - alb@cristal.org'/
     -        1x,'=============================================
     -===================='//
     -        1x,'Using generic filename : ',A)
      CALL DATN(DATENOW,TIMENOW)
      CALL CPU_TIME(time_begin)
      write(10,*)
      write(10,*)
C   Read title
      read (4,21) title
      write(*,21)title
      write(10,21)title
C
C.....Read space group numbers
C
      read(4,*,END=3503)isp1,isp2
      GO TO 3504
3503  WRITE(10,*)'  Error reading the space group'
      STOP
3504  CONTINUE
      write(10,*)'Space groups examined',isp1,' to',isp2
C
C.....Min and max size of the box, symmetry
C
      read (4,*)npol,icon,npolmin,npolmax
      if(npolmax.gt.nat)npolmax=nat
      if(npolmin.lt.1)npolmin=1
      print *,'Number of different types of polyhedra:',npol
      write(10,*)'Number of different types of polyhedra:',npol
      print *,'Connectivity degree (0 = 3D net):',icon
      write(10,*)'Connectivity degree (0 = 3D net):',icon
      print *,'Min & max N of polyhedra per cell:',npolmin,npolmax
      write(10,*)'Min & max N of polyhedra per cell:',npolmin,npolmax
      read (4,*)(ncop(i),i=1,npol) 
      print *,'Polyhedra coordinations:',(ncop(i),i=1,npol)
      write(10,*)'Polyhedra coordinations:',(ncop(i),i=1,npol)
30    format(' Min and max a, b, c : ',6f9.4)
29    format(' Min and max angles  : ',6f7.2)
C
C   Read atom types, and then distances in distgrinsp.txt
C
      do 4 k=1,npol
      write(*,*)'For polyhedra',k
      write(10,*)'For polyhedra',k
      write(actyp,'(i1)')ncop(k)
      cop(k)=float(ncop(k))
      read(4,85)atyp(k,1),atyp(k,2)
      namt(1:2)=atyp(k,1)(1:2)
      call bigc(namt)
      satyp(k,1)(1:2)=namt(1:2)
      namt(1:2)=atyp(k,2)(1:2)
      call bigc(namt)
      satyp(k,2)(1:2)=namt(1:2)
85    format(2a4,a1)
      tempo='distgrinsp.txt'
      call open_read1(9,tempo)
84    read(9,85,end=90)btyp(k,1),btyp(k,2),bctyp
      if(atyp(k,1).eq.btyp(k,1).and.atyp(k,2).eq.btyp(k,2)
     1.and.actyp.eq.bctyp)then
      write(10,*)
      write(10,*)'Pair distances ',atyp(k,1),atyp(k,2)
      write(10,*)
      write(*,*)
      write(*,*)'Pair distances ',atyp(k,1),atyp(k,2)
      write(*,*)
      do 3 i=1,2
      do 3 j=i,2
      read(9,*)d0(i,j,k),e0(i,j,k),di0(i,j,k)
      write (*,31) i,j,d0(i,j,k),e0(i,j,k),di0(i,j,k)
      write (10,31) i,j,d0(i,j,k),e0(i,j,k),di0(i,j,k)
3     continue
      read(9,*)f0(2,k),g0(2,k)
      write (*,32)f0(2,k),g0(2,k)
      write (10,32)f0(2,k),g0(2,k)
      write(10,*)
      write(*,*)
C
C   Squared minimum/maximum/ideal distances
C
      f0(1,k)=d0(1,1,k)
      g0(1,k)=e0(1,1,k)
      f(1,k)=f0(1,k)**2
      g(1,k)=g0(1,k)**2
      f(2,k)=f0(2,k)**2
      g(2,k)=g0(2,k)**2
      f3(k)=f(1,k)*4.
      g3(k)=g(1,k)*4.
      e022(k)=e0(2,2,k)*1.5
      if(ncop(k).eq.6)e022(k)=e0(2,2,k)*1.2
      if(ncop(k).eq.5)e022(k)=e0(2,2,k)
      close(9)
      else
      go to 84
      endif
4     continue
31    format(' pair i-j :',i1,'-',i1,' min/max/ideal distances :',3f6.3)
32    format(' second pairs 1-1    min/max distances :',2f6.3)
C
C  Read cell parameters
C
      go to 91
90    print *
      print *,'Problem with atom-types'
	write(10,*)
      write(10,*)'Problem with atom-types'
	STOP
91    continue
      read (4,*)amin,amax,bmin,bmax,cmin,cmax
      print 30,amin,amax,bmin,bmax,cmin,cmax
      write(10,30)amin,amax,bmin,bmax,cmin,cmax
C
C  Framework density min and max
C
      read(4,*)fdmin,fdmax
C
C  Increase the tolerance for the toomuch subroutine
C
      fdmax3=fdmax*2.
C
      print 33,fdmin,fdmax
      write(10,33)fdmin,fdmax
33    format(' Min and max framework density : ',2f6.2)
C
C  Intermediate distances for 2 types of polyhedra
C
      if(npol.gt.1)then
        do k=1,2
        f0(k,3)=(f0(k,1)+f0(k,2))/2.
        f(k,3)=f0(k,3)**2
        g0(k,3)=(g0(k,1)+g0(k,2))/2.
        g(k,3)=g0(k,3)**2
        enddo
      f3(3)=(f(1,1)+f(1,2))*2.
      g3(3)=(g(1,1)+g(1,2))*2.
      e022(3)=(e022(1)+e022(2))/2.
      di0(1,1,3)=(di0(1,1,1)+di0(1,1,2))/2.
      di0(1,2,3)=(di0(1,2,1)+di0(1,2,2))/2.
      di0(2,2,3)=(di0(2,2,1)+di0(2,2,2))/2.
      endif
C
C   Nruns, Ngens, Rdtm, Rdt0
C
      read(4,*)nrun,genmax,rdtm,rdt0m
22    format(' Nrun, genmax,rdtm,rdt0 : ',I8,F10.0,2F6.2)
      write(*,22)nrun,genmax,rdtm,rdt0m
      write(10,22)nrun,genmax,rdtm,rdt0m
      rdt0m2=rdt0m+0.01
      rdt0m3=rdt0m+0.02
      rdt0m4=rdt0m+0.03
      rdt0m5=rdt0m+0.04
C   Cell refinement code
      read(4,*)idls,iref
      print *,' Monte Carlo cycles of distances refinement: ',idls
      write(10,*)' Monte Carlo cycles of distances refinement: ',idls
      if(iref.eq.1)then
      print *,' Cell refined'
      write(10,*)' Cell refined'
      endif
      if(iref.eq.0)then
      print *,' Cell NOT refined'
      write(10,*)' Cell NOT refined'
      endif
C
C   First filename
C             if negative, then print all results
C             not only the new ones
C
         isave=0
      read(4,*)ifname1
         if(ifname1.lt.0)then
            isave=1
            ifname1=-ifname1
         endif
C
      close(4)
      write(10,*)
      write(10,*)
C
C Initialisation
C
      if(ncop(1).gt.4)then
      gen21m=genmax/320.
      gen22m=genmax/160.
      gen23m=genmax/80.
      gen24m=genmax/40.
      gen25m=genmax/20.
      gen26m=genmax/10.
      endif
      if(ncop(1).le.4)then
      gen21m=genmax/80.
      gen22m=genmax/40.
      gen23m=genmax/20.
      gen24m=genmax/10.
      endif
      imin=5000
      igood=0
C  igood for each processor...:
      igoodp=0
C
      iattempt=0
      iatt5=0
      iatt4=0
      iatt3=0
      iatt2=0
      iatt1=0
      iseed=0
      itest=1
      inew=0
      iiit=0
      iiir=0
      igg=1
C      ifname1=ifname1-1
C      call init(iseed)
C
C
C
C     Loop on different space groups -
C           by decreasing symmetry
C
C
C
      do 5003 isp=isp2,isp1,-1
C
C
C... Stop if K is pressed
C
C      TRUN=float(iii)
C      TKILL=MOD(TRUN,50.)
C      IF(TKILL.GE.0..AND.TKILL.LE.2.)THEN
      CALL KILL(PRESSEDK)
      IF(PRESSEDK)GO TO 8900
C      ENDIF
C
      call init(iseed)
          if(iseed.eq.0)iseed=1
          rad=randi(iseed)
C
C      isp=isp2+1
C5004  isp=isp-1
C      if(isp.lt.isp1)go to 5003
C
      write(10,902)
902   format(//80('+')/)
      iatt=0
C
C  Decide for angles and nsym, according to the space group
C
      alpmin=90.
      alpmax=90.
      betmin=90.
      betmax=90.
      gammin=90.
      gammax=90.
      if(isp.ge.195)then
      nsym=1
      print *
      print *,'constraint on symmetry : cubic box'
      write(10,*)
      write(10,*)'constraint on symmetry : cubic box'
      go to 4999
      endif
      if(isp.ge.143)then
      nsym=3
      print *
      print *,'constraint on symmetry : hexagonal/trigonal box'
      write(10,*)
      write(10,*)'constraint on symmetry : hexagonal/trigonal box'
      gammin=120.
      gammax=120.
      go to 4999
      endif
      if(isp.ge.75)then
      nsym=2
      print *
      print *,'constraint on symmetry : tetragonal box'
      write(10,*)
      write(10,*)'constraint on symmetry : tetragonal box'
      go to 4999
      endif
      if(isp.ge.16)then
      nsym=4
      print *
      print *,'constraint on symmetry : orthorhombic box'
      write(10,*)
      write(10,*)'constraint on symmetry : orthorhombic box'
      go to 4999
      endif
      if(isp.ge.3)then
      nsym=5
      print *
      print *,'constraint on symmetry : monoclinic box'
      write(10,*)
      write(10,*)'constraint on symmetry : monoclinic box'
      betmax=120.
      go to 4999
      endif
      if(isp.ge.1)then
      nsym=6
      print *
      print *,'constraint on symmetry : triclinic box'
      write(10,*)
      write(10,*)'constraint on symmetry : triclinic box'
      alpmax=120.
      betmax=120.
      gammax=120.
      go to 4999
      endif
C
C  Angles
C
4999  alpd=alpmax-alpmin
      betd=betmax-betmin
      gamd=gammax-gammin
      print *
      print 29,alpmin,alpmax,betmin,betmax,gammin,gammax
      write(10,*)
      write(10,29)alpmin,alpmax,betmin,betmax,gammin,gammax
C
C     output filename for .cif and .dat
C
      ifname1=isp*10000
C
C---- Find the general position and wyckoff special positions 
C     for that space group inside of the wyckoff.txt file
C
      tempo='wyckoff.txt'
      call open_read1(24,tempo)
C
C   igrw = space group, Hermann Mauguin notation 
C          inside of the Wyckoff.txt file
C
122   read(24,479,end=3502)(igr(ir),ir=1,36),nnsp
479   format(36a1,i3)
      GO TO 3600
3502  WRITE(10,*)'  Error reading the space group'
      STOP
3600  CONTINUE
C
C   npos   = number of Wyckoff positions including the general
C   irl(1) = number of positions in the general
C            (without Bravais : 48 maxi)
C   irl(i>1) = number of positions in the special
C            (without Bravais)
C
      read(24,*)npos
C      write(10,*)'npos  =  ',npos
      do 124 i=1,npos
      read(24,123)irl(i),wy(i)
C      write(10,123)irl(i),wy(i)
      do 124 j=1,irl(i)
      read(24,*)((sm(i,j,k,l),l=1,3),k=1,3),(ts(i,j,k),k=1,3)
C      write(10,*)((sm(i,j,k,l),l=1,3),k=1,3),(ts(i,j,k),k=1,3)
124   continue
123   format(i2,a1)
      if(nnsp.ne.isp)go to 122
      close(24)
C
C Change small characters in capital letters
C
      do 903 ig=1,36
      do 903 ig2=1,7
      if(igr(ig).eq.igr3(ig2))igr(ig)=igr2(ig2)
903   continue
      write(10,478)(igr(ir),ir=1,36),isp
      write(*,478)(igr(ir),ir=1,36),isp
478   format(/' Space Group : ',36a1,'Number : ',i3/)
      write(10,*)
      KXK=IGR(1)
      IF(KXK.EQ.IBRA(1))IBRAVL=1
      IF(KXK.EQ.IBRA(6))IBRAVL=2
      IF(KXK.EQ.IBRA(5))IBRAVL=3
      IF(KXK.EQ.IBRA(7))IBRAVL=4
      IF(KXK.EQ.IBRA(2))IBRAVL=5
      IF(KXK.EQ.IBRA(3))IBRAVL=6
      IF(KXK.EQ.IBRA(4))IBRAVL=7
C
C.....Include the Bravais translation
C
      iabra=ibravl
C
C   Do that for all Wyckoff positions
C
      do 7201 is=1,npos     
      nag(is)=irl(is)
C
C   If Bravais lattice R,A,B,C, then add symmetry with translation 
C                iabra=5,6,7
C        P I F R A B C = 1 2 3 4 5 6 7
C
      go to (7199,7199,7199,7900,7800,7920,7130),iabra
      go to 7199
7900  irl(is)=irl(is)*3
      do 6001 ir=nag(is)+1,irl(is)-nag(is)
      do 6001 i=1,3
      do 6001 j=1,3
      irr=ir-nag(is)
6001  sm(is,ir,i,j)=sm(is,irr,i,j)
      do 6002 ir=2*nag(is)+1,irl(is)
      do 6002 i=1,3
      do 6002 j=1,3
      irr=ir-2*nag(is)
6002  sm(is,ir,i,j)=sm(is,irr,i,j)
      do 6003 ir=nag(is)+1,irl(is)-nag(is)
      irr=ir-nag(is)
      ts(is,ir,1)=ts(is,irr,1)+0.666667
      ts(is,ir,2)=ts(is,irr,2)+0.333333
6003  ts(is,ir,3)=ts(is,irr,3)+0.333333
      do 6004 ir=2*nag(is)+1,irl(is)
      irr=ir-2*nag(is)
      ts(is,ir,1)=ts(is,irr,1)+0.333333
      ts(is,ir,2)=ts(is,irr,2)+0.666667
6004  ts(is,ir,3)=ts(is,irr,3)+0.666667
      go to 7199
7800  irl(is)=irl(is)*2
      do 7801 ir=nag(is)+1,irl(is)
      do 7801 i=1,3
      do 7801 j=1,3
      irr=ir-nag(is)
7801  sm(is,ir,i,j)=sm(is,irr,i,j)
      do 7102 ir=nag(is)+1,irl(is)
      irr=ir-nag(is)
      ts(is,ir,1)=ts(is,irr,1)
      ts(is,ir,2)=ts(is,irr,2)+0.5
7102  ts(is,ir,3)=ts(is,irr,3)+0.5
      go to 7199
7920  irl(is)=irl(is)*2
      do 7802 ir=nag(is)+1,irl(is)
      do 7802 i=1,3
      do 7802 j=1,3
      irr=ir-nag(is)
7802  sm(is,ir,i,j)=sm(is,irr,i,j)
      do 7122 ir=nag(is)+1,irl(is)
      irr=ir-nag(is)
      ts(is,ir,1)=ts(is,irr,1)+0.5
      ts(is,ir,2)=ts(is,irr,2)
7122  ts(is,ir,3)=ts(is,irr,3)+0.5
      go to 7199
7130  irl(is)=irl(is)*2
      do 7131 ir=nag(is)+1,irl(is)
      do 7131 i=1,3
      do 7131 j=1,3
      irr=ir-nag(is)
7131  sm(is,ir,i,j)=sm(is,irr,i,j)
      DO 7132 ir=nag(is)+1,irl(is)
      irr=ir-nag(is)
      ts(is,ir,1)=ts(is,irr,1)+0.5
      ts(is,ir,2)=ts(is,irr,2)+0.5
7132  ts(is,ir,3)=ts(is,irr,3)
7199  continue
691   format(2x,i4)
C
C  Extension of symmetry operators
C
      do 760 ir=1,irl(is)
      do 760 i=1,3
      tt(is,ir,i)=ts(is,ir,i)
      if(tt(is,ir,i).gt.1.)tt(is,ir,i)=tt(is,ir,i)-1.
      do 760 j=1,3
760   smt(is,ir,i,j)=sm(is,ir,i,j)
      nag(is)=irl(is)
C   If Bravais lattice I, F, then add symmetry with translation 
C                iabra = 2,3
C        P I F R A B C = 1 2 3 4 5 6 7
      go to(7200,6500,6600),iabra
      go to 7200
6500  irl(is)=irl(is)*2
      do 6501 ir=nag(is)+1,irl(is)
      do 6501 i=1,3
      do 6501 j=1,3
      irr=ir-nag(is)
6501  smt(is,ir,i,j)=smt(is,irr,i,j)
      do 6502 ir=nag(is)+1,irl(is)
      irr=ir-nag(is)
      tt(is,ir,1)=tt(is,irr,1)+0.5
      if(tt(is,ir,1).gt.1.)tt(is,ir,1)=tt(is,ir,1)-1.
      tt(is,ir,2)=tt(is,irr,2)+0.5
      if(tt(is,ir,2).gt.1.)tt(is,ir,2)=tt(is,ir,2)-1.
      tt(is,ir,3)=tt(is,irr,3)+0.5
      if(tt(is,ir,3).gt.1.)tt(is,ir,3)=tt(is,ir,3)-1.
6502  continue
      go to 7200
6600  irl(is)=irl(is)*4
      do 6601 ir=nag(is)+1,irl(is)-2*nag(is)
      do 6601 i=1,3
      do 6601 j=1,3
      irr=ir-nag(is)
6601  smt(is,ir,i,j)=smt(is,irr,i,j)
      do 6602 ir=2*nag(is)+1,irl(is)-nag(is)
      do 6602 i=1,3
      do 6602 j=1,3
      irr=ir-2*nag(is)
6602  smt(is,ir,i,j)=smt(is,irr,i,j)
      do 6603 ir=3*nag(is)+1,irl(is)
      do 6603 i=1,3
      do 6603 j=1,3
      irr=ir-3*nag(is)
6603  smt(is,ir,i,j)=smt(is,irr,i,j)
      do 6604 ir=nag(is)+1,irl(is)-2*nag(is)
      irr=ir-nag(is)
      tt(is,ir,1)=tt(is,irr,1)+0.5
      if(tt(is,ir,1).gt.1.)tt(is,ir,1)=tt(is,ir,1)-1.
      tt(is,ir,2)=tt(is,irr,2)
      if(tt(is,ir,2).gt.1.)tt(is,ir,2)=tt(is,ir,2)-1.
      tt(is,ir,3)=tt(is,irr,3)+0.5
      if(tt(is,ir,3).gt.1.)tt(is,ir,3)=tt(is,ir,3)-1.
6604  continue
      do 6005 ir=2*nag(is)+1,irl(is)-nag(is)
      irr=ir-2*nag(is)
      tt(is,ir,1)=tt(is,irr,1)+0.5
      if(tt(is,ir,1).gt.1.)tt(is,ir,1)=tt(is,ir,1)-1.
      tt(is,ir,2)=tt(is,irr,2)+0.5
      if(tt(is,ir,2).gt.1.)tt(is,ir,2)=tt(is,ir,2)-1.
      tt(is,ir,3)=tt(is,irr,3)
      if(tt(is,ir,3).gt.1.)tt(is,ir,3)=tt(is,ir,3)-1.
6005  continue
      do 6006 ir=3*nag(is)+1,irl(is)
      irr=ir-3*nag(is)
      tt(is,ir,1)=tt(is,irr,1)
      if(tt(is,ir,1).gt.1.)tt(is,ir,1)=tt(is,ir,1)-1.
      tt(is,ir,2)=tt(is,irr,2)+0.5
      if(tt(is,ir,2).gt.1.)tt(is,ir,2)=tt(is,ir,2)-1.
      tt(is,ir,3)=tt(is,irr,3)+0.5
      if(tt(is,ir,3).gt.1.)tt(is,ir,3)=tt(is,ir,3)-1.
6006  continue
7200  continue
C
C  nas= total number of symmetry operators 
C
      nas(is)=irl(is)
      if(is.eq.1)then
      write(10,*)
      write(10,*)'General position - Wyckoff symbol : ',wy(is)
      write(10,*)
      else
      write(10,*)
      write(10,*)'Special position - Wyckoff symbol : ',wy(is)
      write(10,*)
      endif
      write(10,*)' Total number of symmetry operators = ',nas(is)
      write(10,*)'   Full list of symmetry operators :'
      write(10,1611)
      do 803 ir=1,nas(is)
C
      do 125 i=1,3
      if(tt(is,ir,i).gt.0.5)tt(is,ir,i)=tt(is,ir,i)-1.
125   continue
C
      write(10,152)ir,((smt(is,ir,i,j),j=1,3),i=1,3),(tt(is,ir,i),i=1,3)
  803 continue
7201  continue
      write(10,*)
      write(10,*)
   49 FORMAT(1H 30X,'AND THE EQUIV. POS. ARE:')
  152 FORMAT(' POS.',I3,2X,9F5.1,3F8.4)
 1611 FORMAT(24X,15HROTATION MATRIX,18X,18HTRANSLATION VECTOR)
C 
C
C     END of space group definition
C
C
C     BIG LOOP - BIG LOOP - BIG LOOP - BIG LOOP
C     BIG LOOP - BIG LOOP - BIG LOOP - BIG LOOP
C
C     Loop on different runs
C
C
C$OMP PARALLEL DEFAULT(PRIVATE)
C$OMP& SHARED(title,isp1,isp2,npol,icon,npolmin,npolmax,ncop,
C$OMP& cop,namt,satyp,atyp,d0,e0,di0,f0,g0,f,g,f3,g3,e022,
C$OMP& amin,amax,bmin,bmax,cmin,cmax,fdmin,fdmax,fdmax3,nrun,
C$OMP& genmax,rdtm,rdt0m,rdt0m2,rdt0m3,rdt0m4,rdt0m5,idls,
C$OMP& iref,isave,gen21m,gen22m,gen23m,gen24m,gen25m,
C$OMP& gen26m,imin,iattempt,iatt5,iatt4,iatt3,iatt2,iatt1,
C$OMP& iatt,igood,itest,inew,iiit,iiir,igg,rho,sgmem,rd,rdp,
C$OMP& nt,amem,bmem,cmem,alpmem,betmem,gammem,vmem,nmem,
C$OMP& nmcmem,namemem,mntype,mjmax,mns,mnr,newcos,ident,
C$OMP& formem,ide,file,lfile,igr2,igr3,isys,ibra,pi2,zeot,
C$OMP& rdzeot,sgzeot,nzeot,nmzeot,coseq,ndat,version,datenow,
C$OMP& timenow,time_begin,ifname1,
C$OMP& isp,alpmin,alpmax,betmin,betmax,gammin,gammax,
C$OMP& nsym,alpd,betd,gamd,igr,nnsp,npos,irl,wy,
C$OMP& nag,tt,smt,nas)
C$OMP& FIRSTPRIVATE(iseed,igoodp)
C$OMP DO
      do 5001 iii=1,nrun
C
C     reinit every 1000 runs
C
C      ireinit=mod(iii,1000)
C      if(ireinit.eq.0) call init(iseed,iii)
C
C  Check if iseed already used in a previous successful proposal
C
      iseed=(iseed+iii)*2+1
      ii=igood+1
      init0(ii)=iseed
	if(ii.ge.(igg+1))then
1199   do i=igg,ii-1
         if(init0(ii).eq.init0(i))then
          call init(iseed)
            test=randi(iseed)
            iseed=iseed+igood*10+1
          init0(ii)=iseed
          go to 1199
         endif
       enddo
      endif
C
C  mv2tot will determine the selection of a model
C      it is the sum of M neighbours around the X atoms
C      we test for corner sharing only so that a model
C      is correct if each X has 2 M neighbours (MX2 for
C      tetrahedra or MX3 for octahedra)
C          dividing mv2tot by nl(2) should provide 2... 
C
      mv2tot=0
      iresult=7000
C
      gen=0.
      gen21=0.
      gen22=0.
      gen23=0.
      gen24=0.
      gen25=0.
      gen26=0.
C
C     define the cell parameters
C
      a=(amax-amin)*randi(iseed)+amin
      if(nsym.eq.1)then
      b=a
      c=a
      go to 8000
      endif
      c=(cmax-cmin)*randi(iseed)+cmin
      if(nsym.eq.2.or.nsym.eq.3)then
      b=a
      go to 8000
      endif
      b=(bmax-bmin)*randi(iseed)+bmin
C
8000  continue
      if(nsym.le.5)then
      alp=90.
      bet=90.
      gam=90.
      if(nsym.eq.3)gam=120.
      if(nsym.eq.5)bet=betmin+betd*randi(iseed)
      go to 8500
      endif
      alp=alpmin+alpd*randi(iseed)
      bet=betmin+betd*randi(iseed)
      gam=gammin+gamd*randi(iseed)
C     
8500  continue
      DO 8501 I=1,3
      DO 8501 J=1,3
8501  AL(I,J)=0.0
      celpre(1)=a
      celpre(2)=b
      celpre(3)=c
      celpre(4)=alp
      celpre(5)=bet
      celpre(6)=gam
      CALL DCELL(CELPRE,AL,V)
      a0=a
      b0=b
      c0=c
      alp0=alp
      bet0=bet
      gam0=gam
C      write(10,*)'  cell = ',a,b,c,alp,bet,gam
C
C   dslow is for keeping the max move of an atom to 
C               + or - 0.2 A  maximum
C               during the distances improvement
C
      dslowa=0.4/a
      dslowb=0.4/b
      dslowc=0.4/c
C
C    Squared cell parameters
C
      a2=a*a
      b2=b*b
      c2=c*c
*
*     Calculate METRIC matrix
*
      ALPH=CELPRE(4)*PI2/360.
      BETA=CELPRE(5)*PI2/360.
      GAMM=CELPRE(6)*PI2/360.
      MET(1,1)=a2
      MET(2,2)=b2
      MET(3,3)=c2
      MET(1,2)=2.*A*B*COS(GAMM)
      MET(1,3)=2.*A*C*COS(BETA)
      MET(2,3)=2.*B*C*COS(ALPH)
C
      ee=e0(1,1,1)
	if(npol.eq.2)then
      if(e0(1,1,2).gt.ee)ee=e0(1,1,2)
	endif
      xa=1./a*ee+0.05
      xa2=2.*xa
      xb=1./b*ee+0.05
      xb2=2.*xb
      xc=1./c*ee+0.05
      xc2=2.*xc
C  
C  Decide which positions would generate too much T atoms
C             and eliminate them
C
      itot=0
      call toomuch(itot,npos,nas,v,fdmax3,np2,np1,pp1)
      if(np2.eq.0)go to 5002
C
      do 4501 ik1=1,maxa
      do 4501 ik2=1,nmax
      mv(ik1,ik2)=0
      do 4501 ik3=1,maxc
      nv(ik1,ik2,ik3)=0
4501  continue
C
C Place first atom of first type (T1)
C
4502  itot=0
      nl(1)=1
      nl(2)=0
C
C Decide for the polyhedra type
C
      mcop(nl(1))=int(randi(iseed)*float(npol)+1.)
      if(mcop(nl(1)).le.0)mcop(nl(1))=1
      if(mcop(nl(1)).gt.npol)mcop(nl(1))=npol
C
C  Decide for the Wyckoff position selected between np1 and npos
C
      mwyc(nl(1))=int(randi(iseed)*float(npos-np1)+pp1)
      if(mwyc(nl(1)).le.0)then
      write(10,*)'4502 mwyc,npos,np1,pp1 ',mwyc(nl(1)),npos,np1,pp1
      mwyc(nl(1))=1
      endif
C
      x0=randi(iseed)
      y0=randi(iseed)
      z0=randi(iseed)
      gen=gen+1.
      if(gen.gt.genmax)go to 5002
C
C Extend to all positions corresponding to mwyc(nl(1))
C
      is=mwyc(nl(1))
      it0=itot+1
      it01=it0-1+nas(is)
      if(it01.gt.npolmax)go to 4502
      DO 299 k=1,nas(is)
      itot=itot+1
      it1(itot)=it0
      it2(itot)=it01
      ist(itot)=is
      mcop(itot)=mcop(nl(1))
      x(itot,1,1)=x0*smt(is,k,1,1)+y0*smt(is,k,1,2)+z0*smt(is,k,1,3)
     1+tt(is,k,1)
      x(itot,1,2)=x0*smt(is,k,2,1)+y0*smt(is,k,2,2)+z0*smt(is,k,2,3)
     1+tt(is,k,2)
      x(itot,1,3)=x0*smt(is,k,3,1)+y0*smt(is,k,3,2)+z0*smt(is,k,3,3)
     1+tt(is,k,3)
      if(x(itot,1,1).gt.1.)x(itot,1,1)=x(itot,1,1)-1.
      if(x(itot,1,2).gt.1.)x(itot,1,2)=x(itot,1,2)-1.
      if(x(itot,1,3).gt.1.)x(itot,1,3)=x(itot,1,3)-1.
      if(x(itot,1,1).lt.0.)x(itot,1,1)=x(itot,1,1)+1.
      if(x(itot,1,2).lt.0.)x(itot,1,2)=x(itot,1,2)+1.
      if(x(itot,1,3).lt.0.)x(itot,1,3)=x(itot,1,3)+1.
C      write(10,*)x(itot,1,1),x(itot,1,2),x(itot,1,3)
299   continue
C
C  Now we have itot atoms already...
C
C
C  Avoid short distances
C
      if(itot.eq.1)go to 302
      do 300 mm1=1,itot-1
      do 300 mm2=mm1+1,itot
        p1=x(mm1,1,1)-x(mm2,1,1)
        q1=x(mm1,1,2)-x(mm2,1,2)
        r1=x(mm1,1,3)-x(mm2,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(1,mcop(1))) go to 4502
300   continue
302   continue
      xs(nl(1))=x(it1(itot),1,1)
      ys(nl(1))=x(it1(itot),1,2)
      zs(nl(1))=x(it1(itot),1,3)
      nass(nl(1))=nas(is)
      wys(nl(1))=wy(is)
C
55    format('  nl(1),gen,x,y,z ',i3,f12.0,3f7.4,2i3)
C
C  We expect to have each T surrounded by 3, 4, 5 or 6T at first
C     distance, then the total of expected neighbours is :
C
      mvexp=ncop(mcop(nl(1)))*itot
C
C  See neighbouring -
C  Are some T atoms already completed ???
C
      call complet(itot,x,met,f,g,nv,mv,mv2tot,ncop,mcop,ibad)
C      if(ibad.eq.1)go to 4502
C
C
C  If mvexp-mv2tot = icon then store the result
C
      iresult=mvexp-mv2tot
      if(iresult.eq.icon)go to 1002
C      if(iresult.lt.imin)then
C        imin=iresult 
C        if(imin.ge.0)print *,'Run, Minimum attained = ',iii,imin
C      endif
C
C Place the next atom of type T at acceptable distances 
C       considering first and second T-T neighbours
C
C  
C  Decide first which positions would generate too much T atoms
C             and eliminate them
C
      call toomuch(itot,npos,nas,v,fdmax3,np2,np1,pp1)
      if(np2.eq.0)go to 5002
C
C Decide for the polyhedra type
C
      mcop(itot+1)=int(randi(iseed)*float(npol)+1.)
      if(mcop(itot+1).le.0)mcop(itot+1)=1
      if(mcop(itot+1).gt.npol)mcop(itot+1)=npol
      kt=mcop(itot)
      if(kt.ne.mcop(itot+1))kt=3
C
      gen21=0.   
C
C  Decide for the Wyckoff position selected between np1 and npos
C
499   mwyc(nl(1)+1)=int(randi(iseed)*float(npos-np1)+pp1)
      if(mwyc(nl(1)+1).le.0)then
      write(10,*)'499 mwyc,npos,np1,pp1 ',mwyc(nl(1)+1),npos,np1,pp1
      mwyc(nl(1)+1)=1
      endif
      p=(x(1,1,1)-xa)+xa2*randi(iseed)
      q=(x(1,1,2)-xb)+xb2*randi(iseed)
      r=(x(1,1,3)-xc)+xc2*randi(iseed)
      gen=gen+1.
      gen21=gen21+1.
      if(gen.gt.genmax)go to 5002
C
C Extend to all positions corresponding to mwyc(nl(1))
C
      is=mwyc(nl(1)+1)
      it0=itot+1
      it01=it0-1+nas(is)
      if(it01.gt.npolmax)go to 499
      DO 493 k=1,nas(is)
      itot=itot+1
      it1(itot)=it0
      it2(itot)=it01
      ist(itot)=is
      mcop(itot)=mcop(it0)
      x(itot,1,1)=p*smt(is,k,1,1)+q*smt(is,k,1,2)+r*smt(is,k,1,3)
     1+tt(is,k,1)
      x(itot,1,2)=p*smt(is,k,2,1)+q*smt(is,k,2,2)+r*smt(is,k,2,3)
     1+tt(is,k,2)
      x(itot,1,3)=p*smt(is,k,3,1)+q*smt(is,k,3,2)+r*smt(is,k,3,3)
     1+tt(is,k,3)
C      if(x(itot,1,1).gt.1.)x(itot,1,1)=x(itot,1,1)-1.
C      if(x(itot,1,2).gt.1.)x(itot,1,2)=x(itot,1,2)-1.
C      if(x(itot,1,3).gt.1.)x(itot,1,3)=x(itot,1,3)-1.
C      if(x(itot,1,1).lt.0.)x(itot,1,1)=x(itot,1,1)+1.
C      if(x(itot,1,2).lt.0.)x(itot,1,2)=x(itot,1,2)+1.
C      if(x(itot,1,3).lt.0.)x(itot,1,3)=x(itot,1,3)+1.
493   CONTINUE
C
C  Now we have itot atoms already...
C
C  Examine if one of the new atoms is close enough to one 
C  of the very first ones
C
       itest=0  
       itotal=0
       do 495 it0=it1(1),it2(1)
       do 495 it=it1(itot),it2(itot)
       itotal=itotal+1
        p1=x(it0,1,1)-x(it,1,1)
        q1=x(it0,1,2)-x(it,1,2)
        r1=x(it0,1,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(1,kt).or.rr.gt.g(1,kt))then
        itest=itest+1
        endif
495   continue
        if(itest.eq.itotal)then
        if(gen21.gt.gen21m)go to 4502
         itot=itot-nas(is)
         go to 499
        endif
C
C  Avoid short distances
C
      mm3=it1(itot)
      do 303 mm1=1,itot-1
      kk=mcop(mm1)
      if(mm1.ge.it1(itot))mm3=mm1+1
      do 303 mm2=mm3,itot
C      if(mm1.eq.mm2)go to 303
      if(kk.ne.mcop(mm2))kk=3
        p1=x(mm1,1,1)-x(mm2,1,1)
        q1=x(mm1,1,2)-x(mm2,1,2)
        r1=x(mm1,1,3)-x(mm2,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(1,kk))then
        itot=itot-nas(is)
        go to 499
        endif
303   continue
      nl(1)=nl(1)+1
      xs(nl(1))=x(it1(itot),1,1)
      ys(nl(1))=x(it1(itot),1,2)
      zs(nl(1))=x(it1(itot),1,3)
      nass(nl(1))=nas(is)
      wys(nl(1))=wy(is)
C
C  We expect to have each T surrounded by 4T or 6T at first
C     distance, then the total of neighbours is ncop(1)*T
C
      mvexp=mvexp+ncop(mcop(itot))*nas(is)
C
C  See neighbouring -
C  Are some T atoms already completed ???
C
      call complet(itot,x,met,f,g,nv,mv,mv2tot,ncop,mcop,ibad)
      if(ibad.eq.1)go to 4502
C
C
C  If mvexp-mv2tot = icon then store the result
C
      iresult=mvexp-mv2tot
      if(iresult.eq.icon)go to 1002
C      if(iresult.lt.imin)then
C        imin=iresult 
C        if(imin.ge.0)print *,'Run, Minimum attained = ',iii,imin
C      endif
C
C  Here starts the 3500 loop
C
C  Select randomly a T atom for adding its next neighbour
C
3500  m=int(randi(iseed)*float(itot)+1.)
      if(m.le.0)then
      write(10,*)'3500 m,itot ',m,itot
      endif
      memo=m
C  
C  Decide first which positions would generate too much T atoms
C             and eliminate them
C
      call toomuch(itot,npos,nas,v,fdmax3,np2,np1,pp1)
      if(np2.eq.0)go to 5002
C
C  Incomplete shell ?? 
C
C
C      gen=gen+1.
C      if(gen.gt.genmax)go to 5002
C      go to 3500
C      endif
C
C Decide for the polyhedra type
C
      mcop(itot+1)=int(randi(iseed)*float(npol)+1.)
      if(mcop(itot+1).le.0)mcop(itot+1)=1
      if(mcop(itot+1).gt.npol)mcop(itot+1)=npol
C
C
C  Determine how many neighbours ? And according to that,
C  select the appropriate treatment :
C  1- if ncop(1)=4 then
C   if mv(m,1)=4  : atom already completed
C   if mv(m,1)=3, 2, or 1  : one atom to add in correct position
C  2- if ncop(1)=6 then
C   if mv(m,1)=6  : atom already completed
C   if mv(m,1)=5, 4, 3, 2, or 1 : one atom to add in correct position
C
      if(ncop(mcop(m)).eq.3)go to (3001,3002,3500)mv(m,1)
      if(ncop(mcop(m)).eq.4)go to (3001,3002,3003,3500)mv(m,1)
      if(ncop(mcop(m)).eq.5)go to (3001,3002,3003,3004,3500)mv(m,1)
      if(ncop(mcop(m)).eq.6)go to (3001,3002,3003,3004,3005,3500)mv(m,1)
C
C  Case with only one previous neighbour
C
3001  continue
C
      gen22=0.   
C
C  Decide for the Wyckoff position selected between np1 and npos
C
600   mwyc(itot+1)=int(randi(iseed)*float(npos-np1)+pp1)
      if(mwyc(itot+1).le.0)then
      write(10,*)'600 mwyc,npos,np1,pp1 ',mwyc(itot+1),npos,np1,pp1
      mwyc(itot+1)=1
      endif
      p=(x(m,1,1)-xa)+xa2*randi(iseed)
      q=(x(m,1,2)-xb)+xb2*randi(iseed)
      r=(x(m,1,3)-xc)+xc2*randi(iseed)
      gen=gen+1.
      gen22=gen22+1.
      if(gen.gt.genmax)go to 5002
C
C Extend to all positions corresponding to mwyc(nl(1))
C
      is=mwyc(itot+1)
      it0=itot+1
      it01=it0-1+nas(is)
      if(it01.gt.npolmax)go to 600
      do 593 k=1,nas(is)
      itot=itot+1
      it1(itot)=it0
      it2(itot)=it01
      ist(itot)=is
      mcop(itot)=mcop(it0)
      x(itot,1,1)=p*smt(is,k,1,1)+q*smt(is,k,1,2)+r*smt(is,k,1,3)
     1+tt(is,k,1)
      x(itot,1,2)=p*smt(is,k,2,1)+q*smt(is,k,2,2)+r*smt(is,k,2,3)
     1+tt(is,k,2)
      x(itot,1,3)=p*smt(is,k,3,1)+q*smt(is,k,3,2)+r*smt(is,k,3,3)
     1+tt(is,k,3)
C      if(x(itot,1,1).gt.1.)x(itot,1,1)=x(itot,1,1)-1.
C      if(x(itot,1,2).gt.1.)x(itot,1,2)=x(itot,1,2)-1.
C      if(x(itot,1,3).gt.1.)x(itot,1,3)=x(itot,1,3)-1.
C      if(x(itot,1,1).lt.0.)x(itot,1,1)=x(itot,1,1)+1.
C      if(x(itot,1,2).lt.0.)x(itot,1,2)=x(itot,1,2)+1.
C      if(x(itot,1,3).lt.0.)x(itot,1,3)=x(itot,1,3)+1.
593   continue
C
      do 700 j=1,2
      if(j.eq.1)mm=m
      if(j.eq.2)mm=nv(m,1,1)
      if(mm.eq.0)go to 3500
      kt=mcop(mm)
      if(j.eq.2)kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
        itest=0
        do 699 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt))then
        itest=itest+1
        endif
699     continue
        if(itest.eq.nas(is))then
        itot=itot-nas(is)
        if(gen22.gt.gen22m)go to 3500
        go to 600
        endif
700   continue
C
C  Avoid short distances
C
      mm3=it1(itot)
      do 800 mm1=1,itot-1
      kk=mcop(mm1)
      if(mm1.ge.it1(itot))mm3=mm1+1
      do 800 mm2=mm3,itot
C      if(mm1.eq.mm2)go to 800
      if(kk.ne.mcop(mm2))kk=3
        p1=x(mm1,1,1)-x(mm2,1,1)
        q1=x(mm1,1,2)-x(mm2,1,2)
        r1=x(mm1,1,3)-x(mm2,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(1,kk))then
        itot=itot-nas(is)
        go to 600
        endif
800   continue
      nl(1)=nl(1)+1
      xs(nl(1))=x(it1(itot),1,1)
      ys(nl(1))=x(it1(itot),1,2)
      zs(nl(1))=x(it1(itot),1,3)
      nass(nl(1))=nas(is)
      wys(nl(1))=wy(is)
C  
      mvexp=mvexp+ncop(mcop(itot))*nas(is)
C
C  Are some T atoms already completed ???
C
      call complet(itot,x,met,f,g,nv,mv,mv2tot,ncop,mcop,ibad)
      if(ibad.eq.1)go to 4502
C
C
C  If mvexp-mv2tot = icon then store the result
C
      iresult=mvexp-mv2tot
      if(iresult.eq.icon)go to 1002
C      if(iresult.lt.imin)then
C        imin=iresult 
C        if(imin.ge.0)print *,'Run, Minimum attained = ',iii,imin
C      endif
C
C  Or go again in the loop 3500
C
      go to 3500
C
C  Case with already 2 previous neighbours
C
3002  continue
      gen23=0.
C   
604   mwyc(itot+1)=int(randi(iseed)*float(npos-np1)+pp1)
      if(mwyc(itot+1).le.0)then
      write(10,*)'604 mwyc,npos,np1,pp1 ',mwyc(itot+1),npos,np1,pp1
      mwyc(itot+1)=1
      endif
      p=(x(m,1,1)-xa)+xa2*randi(iseed)
      q=(x(m,1,2)-xb)+xb2*randi(iseed)
      r=(x(m,1,3)-xc)+xc2*randi(iseed)
      gen=gen+1.
      gen23=gen23+1.
      if(gen.gt.genmax)go to 5002
C
C Extend to all positions corresponding to mwyc(nl(1))
C
      is=mwyc(itot+1)
      it0=itot+1
      it01=it0-1+nas(is)
      if(it01.gt.npolmax)go to 604
      do 693 k=1,nas(is)
      itot=itot+1
      it1(itot)=it0
      it2(itot)=it01
      ist(itot)=is
      mcop(itot)=mcop(it0)
      x(itot,1,1)=p*smt(is,k,1,1)+q*smt(is,k,1,2)+r*smt(is,k,1,3)
     1+tt(is,k,1)
      x(itot,1,2)=p*smt(is,k,2,1)+q*smt(is,k,2,2)+r*smt(is,k,2,3)
     1+tt(is,k,2)
      x(itot,1,3)=p*smt(is,k,3,1)+q*smt(is,k,3,2)+r*smt(is,k,3,3)
     1+tt(is,k,3)
C      if(x(itot,1,1).gt.1.)x(itot,1,1)=x(itot,1,1)-1.
C      if(x(itot,1,2).gt.1.)x(itot,1,2)=x(itot,1,2)-1.
C      if(x(itot,1,3).gt.1.)x(itot,1,3)=x(itot,1,3)-1.
C      if(x(itot,1,1).lt.0.)x(itot,1,1)=x(itot,1,1)+1.
C      if(x(itot,1,2).lt.0.)x(itot,1,2)=x(itot,1,2)+1.
C      if(x(itot,1,3).lt.0.)x(itot,1,3)=x(itot,1,3)+1.
693   continue
C
      do 704 j=1,2
      if(j.eq.1)mm=memo
      if(j.eq.2)mm=nv(m,1,1)
      if(mm.eq.0)go to 3500
      kt=mcop(mm)
      if(j.eq.2)kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
        itest=0
        do 702 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt)) then
           itest=itest+1
        endif
702     continue
         if(itest.eq.nas(is))then
          itot=itot-nas(is)
          go to 604
        endif
      if(j.eq.2)then
      mm=nv(m,1,2)
      if(mm.eq.0)go to 3500
      kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
        itest=0
        do 703 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt))then
           itest=itest+1
	endif
703     continue
        if(itest.eq.nas(is))then
          itot=itot-nas(is)
          if(gen23.gt.gen23m)go to 3500
          go to 604
        endif
      endif
704   continue
C
C  Avoid short distances
C
      mm3=it1(itot)
      do 804 mm1=1,itot-1
      if(mm1.ge.it1(itot))mm3=mm1+1
      kk=mcop(mm1)
      do 804 mm2=mm3,itot
C      if(mm1.eq.mm2)go to 804
      if(kk.ne.mcop(mm2))kk=3
        p1=x(mm1,1,1)-x(mm2,1,1)
        q1=x(mm1,1,2)-x(mm2,1,2)
        r1=x(mm1,1,3)-x(mm2,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(1,kk))then
        itot=itot-nas(is)
          go to 604
         endif
804   continue
      nl(1)=nl(1)+1
      xs(nl(1))=x(it1(itot),1,1)
      ys(nl(1))=x(it1(itot),1,2)
      zs(nl(1))=x(it1(itot),1,3)
      nass(nl(1))=nas(is)
      wys(nl(1))=wy(is)
C
      mvexp=mvexp+ncop(mcop(itot))*nas(is)
C
C  Are some T atoms already completed ???
C
      call complet(itot,x,met,f,g,nv,mv,mv2tot,ncop,mcop,ibad)
      if(ibad.eq.1)go to 4502
C
C  If mvexp-mv2tot = icon then store the result
C
      iresult=mvexp-mv2tot
      if(iresult.eq.icon)go to 1002
C      if(iresult.lt.imin)then
C        imin=iresult 
C        if(imin.ge.0)print *,'Run, Minimum attained = ',iii,imin
C      endif
C
C  Or go again in the loop 3500
C
      go to 3500
C
C  Case with already 3 neighbours 
C
3003  continue
      gen24=0.
C   
605   mwyc(itot+1)=int(randi(iseed)*float(npos-np1)+pp1)
      if(mwyc(itot+1).le.0)then
      write(10,*)'605 mwyc,npos,np1,pp1 ',mwyc(itot+1),npos,np1,pp1
      mwyc(itot+1)=1
      endif
      p=(x(m,1,1)-xa)+xa2*randi(iseed)
      q=(x(m,1,2)-xb)+xb2*randi(iseed)
      r=(x(m,1,3)-xc)+xc2*randi(iseed)
      gen=gen+1.
      gen24=gen24+1.
      if(gen.gt.genmax)go to 5002
C
C Extend to all positions corresponding to mwyc(nl(1))
C
      is=mwyc(itot+1)
      it0=itot+1
      it01=it0-1+nas(is)
      if(it01.gt.npolmax)go to 605
      do 793 k=1,nas(is)
      itot=itot+1
      it1(itot)=it0
      it2(itot)=it01
      ist(itot)=is
      mcop(itot)=mcop(it0)
      x(itot,1,1)=p*smt(is,k,1,1)+q*smt(is,k,1,2)+r*smt(is,k,1,3)
     1+tt(is,k,1)
      x(itot,1,2)=p*smt(is,k,2,1)+q*smt(is,k,2,2)+r*smt(is,k,2,3)
     1+tt(is,k,2)
      x(itot,1,3)=p*smt(is,k,3,1)+q*smt(is,k,3,2)+r*smt(is,k,3,3)
     1+tt(is,k,3)
C      if(x(itot,1,1).gt.1.)x(itot,1,1)=x(itot,1,1)-1.
C      if(x(itot,1,2).gt.1.)x(itot,1,2)=x(itot,1,2)-1.
C      if(x(itot,1,3).gt.1.)x(itot,1,3)=x(itot,1,3)-1.
C      if(x(itot,1,1).lt.0.)x(itot,1,1)=x(itot,1,1)+1.
C      if(x(itot,1,2).lt.0.)x(itot,1,2)=x(itot,1,2)+1.
C      if(x(itot,1,3).lt.0.)x(itot,1,3)=x(itot,1,3)+1.
793   continue
C
      do 705 j=1,2
      if(j.eq.1)mm=memo
      if(j.eq.2)mm=nv(m,1,1)
      if(mm.eq.0)go to 3500
      kt=mcop(mm)
      if(j.eq.2)kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
        itest=0
        do 680 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt)) then
        itest=itest+1
        endif
680   continue
        if(itest.eq.nas(is))then
          itot=itot-nas(is)
          go to 605
        endif
      if(j.eq.2)then
      mm=nv(m,1,2)
      if(mm.eq.0)go to 3500
      kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
      itest=0
        do 681 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt))then
        itest=itest+1
        endif
681     continue
        if(itest.eq.nas(is))then
          itot=itot-nas(is)
        if(gen24.gt.gen24m)go to 3500
        go to 605
        endif
      mm=nv(m,1,3)
      if(mm.eq.0)go to 3500
      kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
      itest=0
      do 682 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt))then
         itest=itest+1
        endif
682     continue
        if(itest.eq.nas(is))then
          itot=itot-nas(is)
          go to 605
        endif
      endif
705   continue
C
C  Avoid short distances
C
      mm3=it1(itot)
      do 805 mm1=1,itot-1
      if(mm1.ge.it1(itot))mm3=mm1+1
      kk=mcop(mm1)
      do 805 mm2=mm3,itot
C      if(mm1.eq.mm2)go to 805
      if(kk.ne.mcop(mm2))kk=3
        p1=x(mm1,1,1)-x(mm2,1,1)
        q1=x(mm1,1,2)-x(mm2,1,2)
        r1=x(mm1,1,3)-x(mm2,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(1,kk))then
        itot=itot-nas(is)
          go to 605
         endif
805   continue
C
      nl(1)=nl(1)+1
      xs(nl(1))=x(it1(itot),1,1)
      ys(nl(1))=x(it1(itot),1,2)
      zs(nl(1))=x(it1(itot),1,3)
      nass(nl(1))=nas(is)
      wys(nl(1))=wy(is)
C  
      mvexp=mvexp+ncop(mcop(itot))*nas(is)
C
C  Are some T atoms already completed ???
C
      call complet(itot,x,met,f,g,nv,mv,mv2tot,ncop,mcop,ibad)
      if(ibad.eq.1)go to 4502
C
C  If mvexp-mv2tot = icon then store the result
C
      iresult=mvexp-mv2tot
      if(iresult.eq.icon)go to 1002
C      if(iresult.lt.imin)then
C        imin=iresult 
C        if(imin.ge.0)print *,'Run, Minimum attained = ',iii,imin
C      endif
C
C  Or go again in the loop 3500
C
      go to 3500
C
C  Case with already 4 neighbours (only if ncop > 4)
C     it may occur that only 3 are at the expected distance
C     and the fourth is at 2x the first T-T distance 
C
3004  continue
      gen25=0.
C   
606   mwyc(itot+1)=int(randi(iseed)*float(npos-np1)+pp1)
      if(mwyc(itot+1).le.0)then
      write(10,*)'606 mwyc,npos,np1,pp1 ',mwyc(itot+1),npos,np1,pp1
      mwyc(itot+1)=1
      endif
      p=(x(m,1,1)-xa)+xa2*randi(iseed)
      q=(x(m,1,2)-xb)+xb2*randi(iseed)
      r=(x(m,1,3)-xc)+xc2*randi(iseed)
      gen=gen+1.
      gen25=gen25+1.
      if(gen.gt.genmax)go to 5002
C
C Extend to all positions corresponding to mwyc(nl(1))
C
      is=mwyc(itot+1)
      it0=itot+1
      it01=it0-1+nas(is)
      if(it01.gt.npolmax)go to 606
      do 893 k=1,nas(is)
      itot=itot+1
      it1(itot)=it0
      it2(itot)=it01
      ist(itot)=is
      mcop(itot)=mcop(it0)
      x(itot,1,1)=p*smt(is,k,1,1)+q*smt(is,k,1,2)+r*smt(is,k,1,3)
     1+tt(is,k,1)
      x(itot,1,2)=p*smt(is,k,2,1)+q*smt(is,k,2,2)+r*smt(is,k,2,3)
     1+tt(is,k,2)
      x(itot,1,3)=p*smt(is,k,3,1)+q*smt(is,k,3,2)+r*smt(is,k,3,3)
     1+tt(is,k,3)
C      if(x(itot,1,1).gt.1.)x(itot,1,1)=x(itot,1,1)-1.
C      if(x(itot,1,2).gt.1.)x(itot,1,2)=x(itot,1,2)-1.
C      if(x(itot,1,3).gt.1.)x(itot,1,3)=x(itot,1,3)-1.
C      if(x(itot,1,1).lt.0.)x(itot,1,1)=x(itot,1,1)+1.
C      if(x(itot,1,2).lt.0.)x(itot,1,2)=x(itot,1,2)+1.
C      if(x(itot,1,3).lt.0.)x(itot,1,3)=x(itot,1,3)+1.
893   continue
C
      do 706 j=1,2
      if(j.eq.1)mm=memo
      if(j.eq.2)mm=nv(m,1,1)
      if(mm.eq.0)go to 3500
      kt=mcop(mm)
      if(j.eq.2)kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
      itest=0
      itest2=0
      do 683 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt))then 
          itest=itest+1
        if(j.eq.2)then
          if(rr.lt.f3(kt).or.rr.gt.g3(kt))then
           itest2=itest2+1
        endif
        endif
        endif
683     continue
        if(j.eq.1)then
        if(itest.eq.nas(is))then
             itot=itot-nas(is)
             go to 606
             endif
             endif
        if(j.eq.2)then
          if(itest2.eq.nas(is))then
          itot=itot-nas(is)
          go to 606
          endif
        endif
      if(j.eq.2)then
      mm=nv(m,1,2)
      if(mm.eq.0)go to 3500
      kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
       itest=0
       do 684 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt))then
           if(rr.lt.f3(kt).or.rr.gt.g3(kt))then
            itest=itest+1
        endif
        endif
684     continue
        if(itest.eq.nas(is))then
        if(gen25.gt.gen25m)go to 3500
           itot=itot-nas(is)
           go to 606
           endif
      mm=nv(m,1,3)
      if(mm.eq.0)go to 3500
      kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
        itest=0
        do 685 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt))then
           if(rr.lt.f3(kt).or.rr.gt.g3(kt))then
            itest=itest+1
          endif
          endif
685      continue
         if(itest.eq.nas(is))then
           itot=itot-nas(is)
           go to 606
        endif
      mm=nv(m,1,4)
      if(mm.eq.0)go to 3500
      kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
       itest=0
        do 686 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt))then
           if(rr.lt.f3(kt).or.rr.gt.g3(kt))then
             itest=itest+1
                endif
               endif
686     continue
         if(itest.eq.nas(is))then
           itot=itot-nas(is)
           go to 606
           endif
      endif
706   continue
C
C  Avoid short distances
C
      mm3=it1(itot)
      do 806 mm1=1,itot-1
      if(mm1.ge.it1(itot))mm3=mm1+1
      kk=mcop(mm1)
      do 806 mm2=mm3,itot
C      if(mm1.eq.mm2)go to 806
      if(kk.ne.mcop(mm2))kk=3
        p1=x(mm1,1,1)-x(mm2,1,1)
        q1=x(mm1,1,2)-x(mm2,1,2)
        r1=x(mm1,1,3)-x(mm2,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(1,kk))then
        itot=itot-nas(is)
        go to 606
        endif
806   continue
C
      nl(1)=nl(1)+1
      xs(nl(1))=x(it1(itot),1,1)
      ys(nl(1))=x(it1(itot),1,2)
      zs(nl(1))=x(it1(itot),1,3)
      nass(nl(1))=nas(is)
      wys(nl(1))=wy(is)
C  
      mvexp=mvexp+ncop(mcop(itot))*nas(is)
C
C  Are some T atoms already completed ???
C
      call complet(itot,x,met,f,g,nv,mv,mv2tot,ncop,mcop,ibad)
      if(ibad.eq.1)go to 4502
C
C  If mvexp-mv2tot = icon then store the result
C
      iresult=mvexp-mv2tot
      if(iresult.eq.icon)go to 1002
C      if(iresult.lt.imin)then
C        imin=iresult 
C        if(imin.ge.0)print *,'Run, Minimum attained = ',iii,imin
C      endif
C
C  Or go again in the loop 3500
C
      go to 3500
C
C  Case with already 5 neighbours 
C
3005  continue
      gen26=0.
C   
607   mwyc(itot+1)=int(randi(iseed)*float(npos-np1)+pp1)
      if(mwyc(itot+1).le.0)then
      write(10,*)'606 mwyc,npos,np1,pp1 ',mwyc(itot+1),npos,np1,pp1
      mwyc(itot+1)=1
      endif
      p=(x(m,1,1)-xa)+xa2*randi(iseed)
      q=(x(m,1,2)-xb)+xb2*randi(iseed)
      r=(x(m,1,3)-xc)+xc2*randi(iseed)
      gen=gen+1.
      gen26=gen26+1.
      if(gen.gt.genmax)go to 5002
C
C Extend to all positions corresponding to mwyc(nl(1))
C
      is=mwyc(itot+1)
      it0=itot+1
      it01=it0-1+nas(is)
      if(it01.gt.npolmax)go to 607
      do 993 k=1,nas(is)
      itot=itot+1
      it1(itot)=it0
      it2(itot)=it01
      ist(itot)=is
      mcop(itot)=mcop(it0)
      x(itot,1,1)=p*smt(is,k,1,1)+q*smt(is,k,1,2)+r*smt(is,k,1,3)
     1+tt(is,k,1)
      x(itot,1,2)=p*smt(is,k,2,1)+q*smt(is,k,2,2)+r*smt(is,k,2,3)
     1+tt(is,k,2)
      x(itot,1,3)=p*smt(is,k,3,1)+q*smt(is,k,3,2)+r*smt(is,k,3,3)
     1+tt(is,k,3)
C      if(x(itot,1,1).gt.1.)x(itot,1,1)=x(itot,1,1)-1.
C      if(x(itot,1,2).gt.1.)x(itot,1,2)=x(itot,1,2)-1.
C      if(x(itot,1,3).gt.1.)x(itot,1,3)=x(itot,1,3)-1.
C      if(x(itot,1,1).lt.0.)x(itot,1,1)=x(itot,1,1)+1.
C      if(x(itot,1,2).lt.0.)x(itot,1,2)=x(itot,1,2)+1.
C      if(x(itot,1,3).lt.0.)x(itot,1,3)=x(itot,1,3)+1.
993   continue
C
      do 707 j=1,2
      if(j.eq.1)mm=memo
      if(j.eq.2)mm=nv(m,1,1)
      if(mm.eq.0)go to 3500
      kt=mcop(mm)
      if(j.eq.2)kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
        itest=0
        itest2=0
      do 660 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt))then 
           if(j.eq.1)then
           itest=itest+1
           endif
        if(j.eq.2)then
           if(rr.lt.f3(kt).or.rr.gt.g3(kt))then
           itest2=itest2+1
           endif
        endif
        endif
660     continue
           if(j.eq.1)then
           if(itest.eq.nas(i))then
           itot=itot-nas(is)
           go to 607
           endif
        if(j.eq.2)then
           if(itest2.eq.nas(i))then
           itot=itot-nas(is)
           go to 607
           endif
        endif
        endif
      if(j.eq.2)then
      mm=nv(m,1,2)
      if(mm.eq.0)go to 3500
      kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
        itest=0
        do 661 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt))then
           if(rr.lt.f3(kt).or.rr.gt.g3(kt))then
           itest=itest+1
           endif
        endif
661     continue
           if(itest.eq.nas(is))then
           itot=itot-nas(is)
        if(gen26.gt.gen26m)go to 3500
           go to 607
           endif
      mm=nv(m,1,3)
      if(mm.eq.0)go to 3500
      kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
        itest=0
      do 662 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt))then
           if(rr.lt.f3(kt).or.rr.gt.g3(kt))then
           itest=itest+1
           endif
        endif
662    continue
           if(itest.eq.nas(is))then
           itot=itot-nas(is)
           go to 607
           endif
      mm=nv(m,1,4)
      if(mm.eq.0)go to 3500
      kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
        itest=0
      do 663 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt))then
           if(rr.lt.f3(kt).or.rr.gt.g3(kt))then
           itest=itest+1
           endif
        endif
663     continue
           if(itest.eq.nas(is))then
           itot=itot-nas(is)
           go to 607
           endif
      mm=nv(m,1,5)
      if(mm.eq.0)go to 3500
      kt=mcop(ny(mm))
      if(kt.ne.mcop(itot))kt=3
        itest=0
        do 664 it=it1(itot),it2(itot)
        p1=y(mm,1)-x(it,1,1)
        q1=y(mm,2)-x(it,1,2)
        r1=y(mm,3)-x(it,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.f(j,kt).or.rr.gt.g(j,kt))then
           if(rr.lt.f3(kt).or.rr.gt.g3(kt))then
           itest=itest+1
           endif
        endif
664    continue
         if(itest.eq.nas(is))then
           itot=itot-nas(is)
           go to 607
           endif
      endif
707   continue
C
C  Avoid short distances
C
      mm3=it1(itot)
      do 807 mm1=1,itot-1
      if(mm1.ge.it1(itot))mm3=mm1+1
      kk=mcop(mm1)
      do 807 mm2=mm3,itot
      if(mm1.eq.mm2)go to 807
      if(kk.ne.mcop(mm2))kk=3
        p1=x(mm1,1,1)-x(mm2,1,1)
        q1=x(mm1,1,2)-x(mm2,1,2)
        r1=x(mm1,1,3)-x(mm2,1,3)
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
          if(rr.lt.f(1,kk))then
          itot=itot-nas(is)
          go to 607
          endif
807   continue
C
      nl(1)=nl(1)+1
      xs(nl(1))=x(it1(itot),1,1)
      ys(nl(1))=x(it1(itot),1,2)
      zs(nl(1))=x(it1(itot),1,3)
      nass(nl(1))=nas(is)
      wys(nl(1))=wy(is)
C  
      mvexp=mvexp+ncop(mcop(itot))*nas(is)
C
C  Are some T atoms already completed ???
C
      call complet(itot,x,met,f,g,nv,mv,mv2tot,ncop,mcop,ibad)
      if(ibad.eq.1)go to 4502
C
C  If mvexp-mv2tot = icon then store the result
C
      iresult=mvexp-mv2tot
      if(iresult.eq.icon)go to 1002
C      if(iresult.lt.imin)then
C        imin=iresult 
C        if(imin.ge.0)print *,'Run, Minimum attained = ',iii,imin
C      endif
C
C  Or go again in the loop 3500
C
      go to 3500
C
C
C  If going to 1002 : possible solution found...
C
C
1002  continue
C
C  Eliminate solutions with too few polyhedra
C
      if(itot.lt.npolmin)go to 5001
C
C  Eliminate unmixed solutions if mixed are searched
C
      if(npol.gt.1)then
       ikt1=1
       ikt2=1
        do i=1,itot
        kt=mcop(i)
         if(kt.eq.1)then
         ikt1=0
         else
         ikt2=0
         endif
        enddo
       ikt=ikt1+ikt2
       if(ikt.ne.0)go to 5001
      endif
C
      imin=0
      if(iresult.eq.icon)then
         iattempt=iattempt+1
         iatt=iatt+1
         igoodp=igoodp+1
C
C     Store some of the T-T results
C
      do i=1,itot
      do k=1,ncop(mcop(i))
      nvt(i,k)=nv(i,1,k)
      enddo
      enddo
C
C     Prepare optimization
C     Place the X atoms and then refine by Monte Carlo
C    
      imemnl=nl(1)
      nl(1)=itot
C      write(10,*)' before midpt'
      call midpt(x,nv,mv,nl,ncop,mcop)
C      write(10,*)' after midpt'
C 
C     Distances refinement, etc.
C 
C     Rd1 : distances T-X
C
      rd1=0.
      rd1d=0.
      do 7000 i=1,nl(1)
      p=x(i,1,1)
      q=x(i,1,2)
      r=x(i,1,3)
      kt=mcop(i)
      do 7100 j=1,ncop(kt)
        p1=xy(i,1,j)-p
        q1=xy(i,2,j)-q
        r1=xy(i,3,j)-r
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
      rd1=rd1+(sqrt(rr)-di0(1,2,kt))**2
7100  continue
      rd1d=rd1d+float(ncop(kt))*(di0(1,2,kt)*2.)**2
7000  continue
C  weight w=2.
      rd1=rd1*4.
C
C      rd2 : distances X-X intra polyhedra
C
      rd2=0.
      rd2d=0.
      do 7001 i=1,nl(1)
      kt=mcop(i)
      do 7001 j=1,ncop(kt)-1
    	p=xy(i,1,j)
      q=xy(i,2,j)
      r=xy(i,3,j)
      do 7001 k=j+1,ncop(kt)
        p1=xy(i,1,k)-p
        q1=xy(i,2,k)-q
        r1=xy(i,3,k)-r
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
      rr=sqrt(rr)
         if(rr.le.e022(kt))then
         rd2=rd2+(rr-di0(2,2,kt))**2
         rd2d=rd2d+(di0(2,2,kt)*0.61)**2
         endif
7001  continue
C weight w=0.61
      rd2=rd2*0.3721
C
C      rd3 :  distances T-T
C
      rd3=0.
      rd3d=0.
      do 7002 i=1,nl(1)
      p=x(i,1,1)
      q=x(i,1,2)
      r=x(i,1,3)
      do 7002 j=1,ncop(mcop(i))
        kt=mcop(i)
        l=nvt(i,j)
        kp=mcop(ny(l))
        if(kt.ne.kp)kt=3
        p1=y(l,1)-p
        q1=y(l,2)-q
        r1=y(l,3)-r
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
      rd3=rd3+(sqrt(rr)-di0(1,1,kt))**2
      rd3d=rd3d+(di0(1,1,kt)*0.23)**2
7002  continue
C  weight w=0.23
      rd3=rd3*0.0529
C
      rdtt=rd1d+rd2d+rd3d
      rdt=sqrt((rd1+rd2+rd3)/rdtt)
C
      rd01=sqrt(rd1/rd1d)
      rd02=sqrt(rd2/rd2d)
      rd03=sqrt(rd3/rd3d)
      rdt0=rdt
      if(rdt.le.rdt0m5)iatt5=iatt5+1
      if(rdt.le.rdt0m4)iatt4=iatt4+1
      if(rdt.le.rdt0m3)iatt3=iatt3+1
      if(rdt.le.rdt0m2)iatt2=iatt2+1
      if(rdt.le.rdt0m)iatt1=iatt1+1
      if(rdt.ge.rdt0m)then
      igoodp=igoodp-1
      go to 5001
      endif
C
C  Monte Carlo distance improvement
C
C    Loop of idls moves per atom 
C            but move also the cell parameters
C
      improve=0
      imove=0
      nltot=nl(1)+nl(2)
      jmc=idls*nltot
      ajmc=jmc
      jmc3=jmc/4
      do 7500 imc=1,jmc
C
C  Damping
C
      aimc=imc
      damp=0.1+0.9*((ajmc-aimc)/ajmc)**4
C      damp=1
C      if(((ajmc-aimc)/ajmc).lt.0.3)damp=0.1
C
C  Select an atom 
C          i for type 1 or 2
C          m for atom order in the list of either nl(1) or nl(2)
C
C  or select one cell parameter (icel=1)
C
      icel=int(randi(iseed)*2.)
C
C     Do not refine the cell if iref = 0
C        or if imc < jmc3
C
      if(imc.lt.jmc3*2)icel=1
      if(imc.lt.jmc3)icel=0
      if(iref.eq.0)icel=0
C
C     change a cell parameter a or b or c by (+ or -) 0.015 A max
C     change an angle cell parameter alp, bet or gam
C                             by (+ or -) 0.015  max
C
      if(icel.eq.1)then
      if(nsym.le.4)mc=int(randi(iseed)*3.+1.)
      if(nsym.eq.5)mc=int(randi(iseed)*4.+1.)
      if(nsym.eq.6)mc=int(randi(iseed)*6.+1.)
C
C     redefine the cell parameters
C
      if(mc.eq.1)anew=a+(randi(iseed)-0.5)*0.03*damp
      if(mc.eq.2)bnew=b+(randi(iseed)-0.5)*0.03*damp
      if(mc.eq.3)cnew=c+(randi(iseed)-0.5)*0.03*damp
      if(mc.eq.4)betnew=bet+(randi(iseed)-0.5)*0.03*damp
      if(mc.eq.5)alpnew=alp+(randi(iseed)-0.5)*0.03*damp
      if(mc.eq.6)gamnew=gam+(randi(iseed)-0.5)*0.03*damp
      if(nsym.eq.1)then
      if(mc.eq.1)then
      bnew=anew
      cnew=anew
      endif
      if(mc.eq.2)then
      anew=bnew
      cnew=bnew
      endif
      if(mc.eq.3)then
      anew=cnew
      bnew=cnew
      endif
      alpnew=alp
      betnew=bet
      gamnew=gam
      go to 8001
      endif
      if(nsym.eq.2.or.nsym.eq.3)then
      if(mc.eq.1)then
      bnew=anew
      cnew=c
      endif
      if(mc.eq.2)then
      anew=bnew
      cnew=c
      endif
      if(mc.eq.3)then
      anew=a
      bnew=b
      endif
      alpnew=alp
      betnew=bet
      gamnew=gam
      go to 8001
      endif
C Orthorhombic or more
      if(nsym.ge.4)then
      if(mc.eq.1)then
      bnew=b
      cnew=c
      endif
      if(mc.eq.2)then
      anew=a
      cnew=c
      endif
      if(mc.eq.3)then
      anew=a
      bnew=b
      endif
      if(nsym.eq.4)then
      alpnew=alp
      betnew=bet
      gamnew=gam
      endif
      endif
C  Monoclinic
      if(nsym.eq.5)then
      if(mc.eq.4)then
      alpnew=alp
      gamnew=gam
      else
      alpnew=alp
      betnew=bet
      gamnew=gam
      endif
      go to 8001
      endif
C  Triclinic
      if(nsym.eq.6)then
      if(mc.eq.4)then
      alpnew=alp
      gamnew=gam
      endif
      if(mc.eq.5)then
      betnew=bet
      gamnew=gam
      endif
      if(mc.eq.6)then
      alpnew=alp
      betnew=bet
      endif
      endif
C
C
8001  continue
      DO 8499 I=1,3
      DO 8499 J=1,3
8499  AL(I,J)=0.0
      celpren(1)=anew
      celpren(2)=bnew
      celpren(3)=cnew
      celpren(4)=alpnew
      celpren(5)=betnew
      celpren(6)=gamnew
      CALL DCELL(CELPREN,AL,Vnew)
C
C   dslow is for keeping the max move of an atom to 
C               + or - 0.3 A (maximum - see damp)
C               during the distances improvement
C
C
C    Squared cell parameters
C
      a2new=anew*anew
      b2new=bnew*bnew
      c2new=cnew*cnew
*
*     Calculate new METRIC matrix
*
      ALPHn=alpnew*PI2/360.
      BETAn=betnew*PI2/360.
      GAMMn=gamnew*PI2/360.
      METn(1,1)=a2new
      METn(2,2)=b2new
      METn(3,3)=c2new
      METn(1,2)=2.*Anew*Bnew*COS(GAMMn)
      METn(1,3)=2.*Anew*Cnew*COS(BETAn)
      METn(2,3)=2.*Bnew*Cnew*COS(ALPHn)
C
C    or move an atom
C
      else
      i=1
      m=int(randi(iseed)*float(nltot)+1.)
      if(m.gt.nltot)m=nltot
      if(m.gt.nl(1))then
      i=2
      m=m-nl(1)
      endif
C
      endif
C
C  Either cell or atom move :
C
      if(icel.eq.1)then
C
C  Calculated rdnew for changes on cell
C
      rd1new=0.
      do 8002 ij=1,nl(1)
      p=x(ij,1,1)
      q=x(ij,1,2)
      r=x(ij,1,3)
      kt=mcop(ij)
      do 8102 j=1,ncop(kt)
        p1=xy(ij,1,j)-p
        q1=xy(ij,2,j)-q
        r1=xy(ij,3,j)-r
      rr=metn(1,1)*p1*p1+metn(2,2)*q1*q1+metn(3,3)*r1*r1
     1+metn(1,2)*p1*q1+metn(1,3)*p1*r1+metn(2,3)*q1*r1
      rd1new=rd1new+(sqrt(rr)-di0(1,2,kt))**2
8102  continue
8002  continue
C  weight w=2.
      rd1new=rd1new*4.
C
C      distances between Xj-Xk for polyhedron i
C      r2 = R factor on d22 intra polyhedra
C
      rd2new=0.
      do 8003 ij=1,nl(1)
      kt=mcop(ij)
      do 8003 j=1,ncop(kt)-1
    	p=xy(ij,1,j)
      q=xy(ij,2,j)
      r=xy(ij,3,j)
      do 8003 k=j+1,ncop(kt)
        p1=xy(ij,1,k)-p
        q1=xy(ij,2,k)-q
        r1=xy(ij,3,k)-r
      rr=metn(1,1)*p1*p1+metn(2,2)*q1*q1+metn(3,3)*r1*r1
     1+metn(1,2)*p1*q1+metn(1,3)*p1*r1+metn(2,3)*q1*r1
      rr=sqrt(rr)
         if(rr.le.e022(kt))then
         rd2new=rd2new+(rr-di0(2,2,kt))**2
         endif
8003  continue
C weight w=0.61
      rd2new=rd2new*0.3721
C
C      rd3 :  distances T-T
C
      rd3new=0.
      do 8004 ij=1,nl(1)
      p=x(ij,1,1)
      q=x(ij,1,2)
      r=x(ij,1,3)
      do 8004 j=1,ncop(mcop(ij))
        kt=mcop(ij)
        l=nvt(ij,j)
        kp=mcop(ny(l))
        if(kt.ne.kp)kt=3
        p1=y(l,1)-p
        q1=y(l,2)-q
        r1=y(l,3)-r
      rr=metn(1,1)*p1*p1+metn(2,2)*q1*q1+metn(3,3)*r1*r1
     1+metn(1,2)*p1*q1+metn(1,3)*p1*r1+metn(2,3)*q1*r1
      rd3new=rd3new+(sqrt(rr)-di0(1,1,kt))**2
8004  continue
C  weight w=0.23
      rd3new=rd3new*0.0529
C
      else
C
C  Decide for a random move of that atom
C
      dp=(randi(iseed)-0.5)*dslowa*damp
      dq=(randi(iseed)-0.5)*dslowb*damp
      dr=(randi(iseed)-0.5)*dslowc*damp
    	pmc=x(m,i,1)+dp
      qmc=x(m,i,2)+dq
      rmc=x(m,i,3)+dr
      imove=imove+1
C
C  Changes on distances
C
C  Calculate new Rd1, Rd2, Rd3
C
      rd1new=0.
      do 7020 ij=1,nl(1)
      kt=mcop(ij)
      p=x(ij,1,1)
      q=x(ij,1,2)
      r=x(ij,1,3)
        if(i.eq.1)then
          if(ij.eq.m)then
            p=pmc
            q=qmc
            r=rmc
          endif
        endif
C   rd1new
      do 7120 j=1,ncop(kt)
        p1=xy(ij,1,j)-p
        q1=xy(ij,2,j)-q
        r1=xy(ij,3,j)-r
         if(i.eq.2)then
           if(nxy(ij,j).eq.m)then
           p1=p1+dp
           q1=q1+dq
           r1=r1+dr
           endif
        endif
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
      rd1new=rd1new+(sqrt(rr)-di0(1,2,kt))**2
7120  continue
7020  continue
C  weight w=2.
      rd1new=rd1new*4.
C
C      distances between Xj-Xk for polyhedron i
C      r2 = R factor on d22 intra polyhedra
       if(i.eq.1)then
       rd2new=rd2
       go to 7031
       endif
      rd2new=0.
      do 7030 ij=1,nl(1)
      kt=mcop(ij)
      do 7030 j=1,ncop(kt)-1
    	p=xy(ij,1,j)
      q=xy(ij,2,j)
      r=xy(ij,3,j)
          if(nxy(ij,j).eq.m)then
            p=p+dp
            q=q+dq
            r=r+dr
          endif
      do 7030 k=j+1,ncop(kt)
        p1=xy(ij,1,k)-p
        q1=xy(ij,2,k)-q
        r1=xy(ij,3,k)-r
          if(nxy(ij,k).eq.m)then
        p1=p1+dp
        q1=q1+dq
        r1=r1+dr
          endif
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
      rr=sqrt(rr)
          if(rr.le.e022(kt))then
          rd2new=rd2new+(rr-di0(2,2,kt))**2
          endif
7030  continue
C weight w=0.61
      rd2new=rd2new*0.3721
7031  continue
      if(i.eq.2)then
      rd3new=rd3
      go to 7032
      endif
C
C      rd3 :  distances T-T
C
      rd3new=0.
      do 7034 ij=1,nl(1)
      p=x(ij,1,1)
      q=x(ij,1,2)
      r=x(ij,1,3)
          if(ij.eq.m)then
            p=pmc
            q=qmc
            r=rmc
          endif
      do 7034 j=1,ncop(mcop(ij))
        kt=mcop(ij)
        l=nvt(ij,j)
        kp=mcop(ny(l))
        if(kt.ne.kp)kt=3
        p1=y(l,1)-p
        q1=y(l,2)-q
        r1=y(l,3)-r
          if(ny(l).eq.m)then
        p1=p1+dp
        q1=q1+dq
        r1=r1+dr
          endif
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
      rd3new=rd3new+(sqrt(rr)-di0(1,1,kt))**2
7034  continue
C  weight w=0.23
      rd3new=rd3new*0.0529
C
7032  continue
C
      endif
C
C
C  If RdT improved, keep the move...
C
      rdtnew=sqrt((rd1new+rd2new+rd3new)/rdtt)
      if(rdtnew.ge.rdt)go to 7500
      improve=improve+1
C   Move kept, then make all changes...
C
C  changes accepted either on cell or atom moves
C
      if(icel.eq.1)then
C
C  Here modif on cell if accepted
C
      a=anew
      b=bnew
      c=cnew
      alp=alpnew
      bet=betnew
      gam=gamnew
      v=vnew
      a2=a2new
      b2=b2new
      c2=c2new
      MET(1,1)=metn(1,1)
      MET(2,2)=metn(2,2)
      MET(3,3)=metn(3,3)
      MET(1,2)=metn(1,2)
      MET(1,3)=metn(1,3)
      MET(2,3)=metn(2,3)
C
      rd1=rd1new
      rd2=rd2new
      rd3=rd3new
      rdt=rdtnew
C
      else
C
C   changes on coordinates x and Rd1,Rd2 and Rdt
C
      x(m,i,1)=pmc
      x(m,i,2)=qmc
      x(m,i,3)=rmc
C
C   changes on y
C
      if(i.eq.1)then
      do ki=1,27*nl(1)
      if(ny(ki).eq.m)then
      y(ki,1)=y(ki,1)+dp
      y(ki,2)=y(ki,2)+dq
      y(ki,3)=y(ki,3)+dr
      endif
	enddo
	endif
C
C   Also changes on xy if i=2...
C
      if(i.eq.2)then
      do ki=1,nl(1)
      do kj=1,ncop(mcop(ki))
      if(nxy(ki,kj).eq.m)then
      xy(ki,1,kj)=xy(ki,1,kj)+dp
      xy(ki,2,kj)=xy(ki,2,kj)+dq
      xy(ki,3,kj)=xy(ki,3,kj)+dr
      endif
      enddo
	enddo
	endif
C
C   changes on coordinates x and Rd1,Rd2,Rd3 and Rdt
C
      rd1=rd1new
      rd2=rd2new
      rd3=rd3new
      rdt=rdtnew
      endif
C      write(10,*)'i,m ',i,m
C
7500  continue
C
C  End of Monte Carlo distance improvement
C
C  If Rdt > Rdtmax, reject the cell
C
      if(rdt.gt.rdtm)then
      igoodp=igoodp-1
      go to 5001
      endif
C
C   Do not save if the framework density is outside
C        of the expected range
C
      if(v.lt.1.)then
      igoodp=igoodp-1
      go to 5001
      endif
      tn=nl(1)
      rhop=tn/v*1000.
      if(rhop.lt.fdmin)then
      igoodp=igoodp-1
      go to 5001
      endif
      if(rhop.gt.fdmax)then
      igoodp=igoodp-1
      go to 5001
      endif
C
C  Calculate coordination sequence
C
      ibad=0
      call coorseq(nl,x,met,g,ntype,nr,ns,jmax,ncop,mcop,npol,ibad,
     1ncs)
      if(ibad.eq.1)then
      igoodp=igoodp-1
      go to 5001
      endif
C
C  If a first solution is found :
C
C
      if((igood+1).ge.2000)then
      igood=igood-1
      print *,' igood > 2000... type K to stop at next SG'
      go to 5001
      endif
C$OMP CRITICAL(STORE)
      igood=igood+1
      if(igood.eq.1)then
      print *,
     1' Run  SG   MCgen     a       b       c     alpha  beta   gamma
     1  nT    R'

      tempo=file(1:lfile)//'.con'
      inquire(file=tempo,exist=qex)
      if(qex.eq..FALSE.) go to 259
      call filedel(15,tempo)
259   continue
      call open_write1(15,tempo)
C
C     Read the connectivity data only once
C
      tempo='connectivity.txt'
      call open_read1(9,tempo)
C  Read total number of data
      i=0
C  Read name
5305  i=i+1
      if(i.gt.5000)go to 5306
      read(9,5302,end=5306)zeot(i),rdzeot(i),sgzeot(i)
C      write(10,5302)zeot(i),rdzeot(i),sgzeot(i)
C  Read number of T sites for that data
      read(9,*)nzeot(i)
C      write(10,*)nzeot(i)
C  Read multiplicity of T sites for that data
      read(9,*)(nmzeot(k),k=1,nzeot(i))
      do 5300 j=1,nzeot(i)
      read(9,5301)coseq(j,i)(1:40)
C      write(10,*)'connectivity data ',coseq(j,i)(1:40)
5300  continue
      go to 5305
5301  format(40A)
5302  format(A11,21x,f7.4,2x,i3)
5306  i=i-1
      ndat=i
      close(9)
C
C
      endif
C
C
      write(10,*)
      write(10,*)'   SOLUTION Number : ',igood
      rd1=sqrt(rd1/rd1d)
      rd2=sqrt(rd2/rd2d)
      rd3=sqrt(rd3/rd3d)
      write(10,*)
      write(10,*)'gen21-26 values:',gen21,gen22,gen23,gen24,gen25,gen26
      write(10,*)
      write(10,*)'Original cell  ',a0,b0,c0,alp0,bet0,gam0
      write(10,*)'         Rd1, Rd2, Rd3, RdT  '
      write(10,*)rd01,rd02,rd03,rdt0
      write(10,*)' Refined Rd1, Rd2, Rd3, RdT  '
      write(10,*)rd1,rd2,rd3,rdt
      write(10,*)
      write(10,*)
     1' Run  SG  MCgen    a       b       c     alpha  beta   gamma
     1 nT    R'
      print 7550,iii,isp,gen,a,b,c,alp,bet,gam,nl(1),rdt
      write(10,7550)iii,isp,gen,a,b,c,alp,bet,gam,nl(1),rdt
7550  format(i6,1x,i3,f9.0,3F8.3,3f7.2,i3,f8.4)
C
C  Store some details for ordering by Rdt
C
      if(iresult.eq.icon)itest=0
      sgmem(igood)=isp
      rd(igood)=rdt
      rdp(igood)=rdt
      nt(igood)=nl(1)
      amem(igood)=a
      bmem(igood)=b
      cmem(igood)=c
      alpmem(igood)=alp
      betmem(igood)=bet
      gammem(igood)=gam
      vmem(igood)=v
      nmem(igood)=iii
      nmcmem(igood)=int(gen)
      ifname1=ifname1+1
      write(fname,'(i7)')ifname1
      namemem(igood)=fname(1:7)
      mntype(igood)=ntype
      mjmax(igood)=jmax
      rho(igood)=rhop
      do i=1,ntype
      mns(igood,i)=ns(i)
      do j=1,10
      mnr(igood,i,j)=nr(i,j)
      enddo
      enddo
C
C  Try to identify if this is already known or already predicted
C
C
C     Now compare with data in connectivity.txt ...
C
C        prepare the current data
C
      iprint=1
      do i=1,ntype
      do j=1,10
      write(t(j),'(i4)')nr(i,j)
      enddo
      newcos(i,igood)=t(1)//t(2)//t(3)//t(4)//t(5)//t(6)//t(7)//
     1t(8)//t(9)//t(10)
      enddo
C
C     compare only on the real length of the predicted sequence
C     
      l=4*jmax
C
      do 6800 i=1,ndat
C
C        skip if not same ntype
C
      if(nzeot(i).ne.ntype)go to 6800
      isum=0
      do 6700 j=1,ntype
      do 6699 k=1,nzeot(i)
      if(newcos(j,igood)(1:l).eq.coseq(k,i)(1:l))then
      isum=isum+1
      if(isum.eq.ntype)go to 6801
      endif
6699  continue      
6700  continue
6800  continue
C   
C    Nothing found
C
      go to 6810
C
C    Something found
C
6801  continue  
      ident(igood)=zeot(i)
      write(10,*)
      write(10,*)'This is probably ',zeot(i)
      iprint=0
C
C   However, check if R is smaller ?
C
      if(rd(igood).lt.rdzeot(i))then
         rdp(i)=rd(igood)
         inew=inew+1
         write(ident(igood),'(a4,i7)')'PCOD',ifname1
      write(10,*)
      write(10,*)'   but the current R is better...',rd(igood),rdzeot(i)
         iprint=1
         go to 6810
      endif
C
C  but make the output files .cif and .dat, anyway 
C              if this is asked for (isave=1)
C
      if(isave.eq.1)iprint=1
      go to 6820
6810  continue
C
C  Compare also with current list
C
      do 6850 i=1,igood-1
C
C        skip if not same ntype
C
      if(ntype.ne.mntype(i))go to 6850
      isum=0
      jmaxmin=jmax
      if(mjmax(i).lt.jmaxmin)jmaxmin=mjmax(i)
      l=4*jmaxmin
      do 6750 j=1,ntype
      do 6749 k=1,mntype(i)
      if(newcos(j,igood)(1:l).eq.newcos(k,i)(1:l))then
      isum=isum+1
         if(isum.eq.ntype)then
         if(rd(igood).gt.rdp(i))iprint=0
         if(rd(igood).lt.rdp(i))rdp(i)=rd(igood)
         go to 6851
         endif
      go to 6750 
      endif
6749  continue      
6750  continue
6850  continue
C   
C    Nothing found
C
      go to 6860
C
C    Something found
C
6851  continue  
      ident(igood)=ident(i)
      write(10,*)
      write(10,*)'This was found already ',ident(i)
      go to 6820
6860  continue
C
C   If nothing found, this is a new one...
C
      inew=inew+1
      write(ident(igood),'(a4,i7)')'PCOD',ifname1
      write(10,*)
      write(10,*)'Found for the first time ',ident(igood)
C
6820  continue
C$OMP END CRITICAL(STORE)
        if(iprint.eq.0)then
          igood=igood-1
          go to 5001
        endif
C
C$OMP CRITICAL(STORE2)
C
      write(ide(igood),'(a4,i7)')'PCOD',ifname1
C
C  End of checking
C
      write(10,*)
      write(10,*)
      write(10,*)' Cations with Wyckoff positions: '
      write(10,*)
      do 2080 i=1,imemnl
      write(10,2081)i,nass(i),wys(i),xs(i),ys(i),zs(i)
2080  continue
2081  format(i2,3x,i3,a1,2x,3f9.4)
      write(10,*)
      write(10,*)' Reduced atomic coordinates: '
      write(10,*)
      do 2011 i=1,nl(1)
      kt=mcop(i)
      write(10,2004)atyp(kt,1)(1:2),i,x(i,1,1),x(i,1,2),x(i,1,3)
2011  continue
      do 2012 i=1,nl(2)
      write(10,2004)atyp(1,2)(1:2),i,x(i,2,1),x(i,2,2),x(i,2,3)
2012  continue
C
      write(10,*)
      write(10,*)' Coordination sequences'
      write(10,1501)nl(1)
      do 1500 i=1,nl(1)
      write(10,1501)(ncs(i,j),j=1,jmax)
1500  continue
1501  format(11I4)
      write(10,*)
      write(10,*)' Reduced coordination sequences'
      write(10,1501)ntype
      write(10,1501)(ns(i),i=1,ntype)
      do 1511 i=1,ntype
      write(10,1501)(nr(i,j),j=1,jmax)
1511  continue
      write(10,*)
C$OMP END CRITICAL(STORE2)
C
C   Do not print if iprint = 0 
C        or if the framework density is outside
C        of the expected range
C
      if(iprint.eq.0)then
      igood=igood-1
      go to 5001
      endif
C
C
C$OMP CRITICAL(STORE3)
C
      tempo=fname(1:7)//'.dat'
      inquire(file=tempo,exist=qex)
      if(qex.eq..FALSE.) go to 200
      call filedel(3,tempo)
200   continue
      call open_write1(3,tempo)
      tempo=fname(1:7)//'.cif'
      inquire(file=tempo,exist=qex)
      if(qex.eq..FALSE.) go to 202
      call filedel(14,tempo)
202   continue
      call open_write1(14,tempo)
C   
C   STRUVIR STRUVIR STRUVIR STRUPLO STRUPLO STRUPLO
C                   CIF CIF CIF CIF CIF CIF
C
C
      WRITE(3,3800)rd1,rd2,rd3,rdt,a,b,c,alp,bet,gam
3800  FORMAT('TITL  GRINSP 2.00 - 2006 - PCOD'/
     1'TITL   Rd1, Rd2, Rd3, Rdt '4f9.5/
     2'CELL ',3f9.4,3f8.3/
     3'SPGP P 1') 
      WRITE(3,3801)
3801  FORMAT('VIEW 0 0 1    0 0 0')
      WRITE(3,3805)
3805  FORMAT('PARA 1.00 0 1 1 1 1 1 3.00 0.00 0.01')
      DX=-0.75
      DY=-0.75
      DZ=-0.5
      EX=1.-DX
      EY=1.-DY
      EZ=1.-DZ
      WRITE(3,3806)DX,EX,DY,EY,DZ,EZ
3806  FORMAT('XYZR ',3(2F5.2,1X))
      if(ncop(1).eq.4)then
      WRITE(3,3807)di0(1,2,1)
      endif
      if(ncop(2).eq.4)then
      WRITE(3,3807)di0(1,2,2)
      endif
3807  FORMAT('TETR 109.0 20.0 ',f4.2,' 0.10')
      if(ncop(1).eq.6)then
      WRITE(3,3808)di0(1,2,1)
      endif
      if(ncop(2).eq.6)then
      WRITE(3,3808)di0(1,2,2)
      endif
3808  FORMAT('OCTA  90.0 20.0 ',f4.2,' 0.10')
      WRITE(3,3809)
3809  FORMAT('PERS     70.00')
      WRITE(3,3810)
3810  FORMAT('FORM 4')
      if(ncop(1).eq.3.or.ncop(1).eq.5)then
      WRITE(3,3812)e0(1,2,1),satyp(1,1)(1:2)
      endif
      if(ncop(2).eq.3.or.ncop(2).eq.5)then
      WRITE(3,3812)e0(1,2,2),satyp(2,1)(1:2)
      endif
3812  FORMAT('BZRR ',f4.2,1x,a2)
      WRITE(3,3811)
3811  FORMAT('END')
C
C  Establish the formula sum
C
      if(npol.eq.1)then
      write(formula,'(a2,i3,a2,i3)')atyp(1,1)(1:2),nl(1),
     1atyp(1,2)(1:2),nl(2)
      else
      n11=0
      n12=0
      do i=1,nl(1)
      if(mcop(i).eq.1)n11=n11+1
      if(mcop(i).eq.2)n12=n12+1
      enddo
      write(formula,'(a2,i3,a2,i3,a2,i3)')atyp(1,1)(1:2),n11,
     1atyp(2,1),n12,atyp(1,2)(1:2),nl(2)
      endif
C
      call form(formula,tchem,ntchem,id)
C
C
      if(formula(10:10).eq.' ')formula(11:11)=' '
      if(formula(11:11).eq.' ')formula(12:12)=' '
      if(formula(12:12).eq.' ')formula(13:13)=' '
      if(formula(13:13).eq.' ')formula(14:14)=' '
      if(formula(14:14).eq.' ')formula(15:15)=' '
      formem(igood)=formula
      write(14,'(a9,i7)')'data_PCOD',ifname1
4028  format('Structure prediction by GRINSP 2.00 - 2006 (A. Le Bail)'/
     1A15,'PCOD',i7,' R = ',f7.4/'Probable space group: ',36a1)
      write(14,'(a)')'_publ_section_title'
      write(14,'(a)')';'
      write(14,4028)formula,ifname1,rdt,(igr(ir),ir=1,36)
      write(14,'(a)')';'
      write(14,'(a)')'_chemical_formula_sum'//tchem(1:ntchem)
	write(idt,'(i3)')id
      write(14,'(a)')'_cell_formula_units_Z  '//idt
      write(14,'(a,f9.4)')'_cell_length_a       ',a
      write(14,'(a,f9.4)')'_cell_length_b       ',b
      write(14,'(a,f9.4)')'_cell_length_c       ',c
      write(14,'(a,f7.3)')'_cell_angle_alpha    ',alp
      write(14,'(a,f7.3)')'_cell_angle_beta     ',bet
      write(14,'(a,f7.3)')'_cell_angle_gamma    ',gam
      write(14,'(a,f12.2)')'_cell_volume     ',v
      write(14,4030)
4030  format('_symmetry_space_group_name_H-M   "'"P 1"'"')
      write(14,'(a30)')'_symmetry_Int_Tables_number  1'
      write(14,'(a)')'loop_'
      write(14,'(a)')'_symmetry_equiv_pos_as_xyz'
      write(14,'(a)')'x,y,z'
      write(14,'(a)')'loop_'
      write(14,'(a)')'_atom_site_label'
C      write(14,'(a)')'_atom_site_symmetry_multiplicity'
C      write(14,'(a)')'_atom_site_Wyckoff_symbol'
      write(14,'(a)')'_atom_site_fract_x'
      write(14,'(a)')'_atom_site_fract_y'
      write(14,'(a)')'_atom_site_fract_z'
C      write(14,'(a)')'_atom_site_occupancy'
C      write(14,'(a)')'_atom_site_Uiso_or_Biso'
      do 2001 i=1,nl(1)
      kt=mcop(i)
      write(3,2004)satyp(kt,1)(1:2),i,x(i,1,1),x(i,1,2),x(i,1,3)
      if(i.le.9)then
      if(atyp(kt,1)(2:2).eq.' ')then
      write(14,2021)atyp(kt,1)(1:1),i,x(i,1,1),x(i,1,2),x(i,1,3)
      go to 2001
      endif
      write(14,2015)atyp(kt,1)(1:2),i,x(i,1,1),x(i,1,2),x(i,1,3)
      go to 2001
      endif
      if(i.le.99)then
      if(atyp(kt,1)(2:2).eq.' ')then
      write(14,2022)atyp(kt,1)(1:1),i,x(i,1,1),x(i,1,2),x(i,1,3)
      go to 2001
      endif
      write(14,2017)atyp(kt,1)(1:2),i,x(i,1,1),x(i,1,2),x(i,1,3)
      go to 2001
      endif
      if(atyp(kt,1)(2:2).eq.' ')then
      write(14,2023)atyp(kt,1)(1:1),i,x(i,1,1),x(i,1,2),x(i,1,3)
      go to 2001
      endif
      write(14,2019)atyp(kt,1)(1:2),i,x(i,1,1),x(i,1,2),x(i,1,3)
2001  continue
      do 2010 i=1,nl(2)
      write(3,2004)satyp(1,2)(1:2),i,x(i,2,1),x(i,2,2),x(i,2,3)
      if(i.le.9)then
      write(14,2021)atyp(1,2)(1:1),i,x(i,2,1),x(i,2,2),x(i,2,3)
      go to 2010
      endif
      if(i.le.99)then
      write(14,2022)atyp(1,2)(1:1),i,x(i,2,1),x(i,2,2),x(i,2,3)
      go to 2010
      endif
      write(14,2023)atyp(1,2)(1:1),i,x(i,2,1),x(i,2,2),x(i,2,3)
2010  continue
2004  format(a2,1x,i3,3f9.4,2x)
2014  format(a2,i1,3X,3f12.5)
2015  format(a2,i1,3X,3f10.5)
2016  format(a2,i2,3X,3f12.5)
2017  format(a2,i2,3X,3f10.5)
2018  format(a2,i3,3X,3f12.5)
2019  format(a2,i3,3X,3f10.5)
2021  format(a1,i1,3X,3f10.5)
2022  format(a1,i2,3X,3f10.5)
2023  format(a1,i3,3X,3f10.5)
2020  format(a3)
2005  format(4x,a1,2x,i2,3f9.4)
C
      write(14,'(a)')'#'
      write(14,'(a33,36a1)')'#_symmetry_space_group_name_H-M  ',
     1(igr(ir),ir=1,36)
      write(14,'(a30,i3)')'#_symmetry_Int_Tables_number  ',isp
      write(14,'(a)')'#'
      write(14,'(a31)')'#   Starting Wyckoff positions:'
      write(14,'(a)')'#'
      do 8080 i=1,imemnl
      write(14,8081)i,nass(i),wys(i),xs(i),ys(i),zs(i)
8080  continue
8081  format('#   ',i2,3x,i3,a1,2x,3f9.4)
      write(14,'(a)')'#'
      write(14,'(a27)')'#   Coordination Sequence :'
      write(14,'(a)')'#'
4029  format('# ',a11,1x,a15,' R = ',f7.4,2x,i3)
      write(14,4029)ide(igood),formem(igood),rd(igood),sgmem(igood)
      write(14,'(a2,I3)')'# ',mntype(igood)
      write(14,'(a2,20I3)')'# ',(mns(igood,ii),ii=1,mntype(igood))
      do kj=1,mntype(igood)
      write(14,'(a2,10I4)')'# ',(mnr(igood,kj,kl),kl=1,10)
      enddo
      write(14,'(a)')'#'
      write(14,'(a21,F6.2)')'#_framework_density  ',rho(igood)
      write(14,'(a)')'#'
      close(3)
      close(14)
C$OMP END CRITICAL(STORE3)
      endif
C
C  End of the loop on runs
C

5002  continue
5001  continue
C
C$OMP END DO
C$OMP END PARALLEL
C
      iiir=nrun
      CALL DATN(DATENOW,TIMENOW)
      CALL CPU_TIME(time_end)
      totaltime=time_end-time_begin
      write(10,*)
      write(10,*)' Total CPU time elapsed in seconds : ',totaltime
      write(10,*)
      write(10,*)' Total number of runs     : ',iiir
      write(10,*)' Number of structure candidates  : ',iatt
      write(10,*)
      iiit=iiit+iiir
C
C  End of the loop on space groups
C
      igg=igood+1
C      go to 5004
5003  continue
C
C
C
8900  continue
C
C  Output of the summary sorted by increasing Rp
C
      IF(IGOOD.EQ.0)GO TO 6000
C
C   Prepare sorted output 
C
      WRITE(10,*)
      WRITE(10,*)'====================================================
     1==========================='
      WRITE(10,*)'       FINAL LIST OF PROPOSALS, sorted by R :'
      WRITE(10,*)'====================================================
     1==========================='
      WRITE(10,*)
      WRITE(10,*)
      CALL SORT(IGOOD,RD,LL)
      WRITE(10,1999)
1999  FORMAT('      R     NT       Vol     FD       a        b     
     1   c      alpha   beta    gamma     MC      Run    File   Ident?')
      DO 2000 I=1,IGOOD
      J=LL(I)
      if(rho(j).lt.fdmin)then
      WRITE(10,2050)RD(J),NT(J),VMEM(J),RHO(J),amem(j),bmem(j),cmem(j),
     1alpmem(j),betmem(j),gammem(j),nmcmem(j),nmem(j),namemem(j),
     2ident(j)
      go to 2000
      endif
      if(rho(j).gt.fdmax)then
      WRITE(10,2051)RD(J),NT(J),VMEM(J),RHO(J),amem(j),bmem(j),cmem(j),
     1alpmem(j),betmem(j),gammem(j),nmcmem(j),nmem(j),namemem(j),
     2ident(j)
      go to 2000
      endif
      WRITE(10,2002)RD(J),NT(J),VMEM(J),RHO(J),amem(j),bmem(j),cmem(j),
     1alpmem(j),betmem(j),gammem(j),nmcmem(j),nmem(j),namemem(j),
     2ident(j)
2000  CONTINUE
2002  FORMAT(2X,F8.4,I4,F12.2,F8.4,3F9.4,3F8.3,I8,I8,1X,A7,1X,A11)
2050  FORMAT('- ',F8.4,I4,F12.2,F8.4,3F9.4,3F8.3,I8,I8,1X,A7,1X,A11)
2051  FORMAT('+ ',F8.4,I4,F12.2,F8.4,3F9.4,3F8.3,I8,I8,1X,A7,1X,A11)
C
C   Prepare reduced sorted output 
C
      WRITE(10,*)
      WRITE(10,*)'====================================================
     1==========================='
      WRITE(10,*)'       FINAL LIST OF UNIQUE PROPOSALS, sorted by R :'
      WRITE(10,*)'====================================================
     1==========================='
      WRITE(10,*)
      WRITE(10,*)
      WRITE(10,1999)
      ninew=0
      DO 2060 I=1,IGOOD
      J=LL(I)
      if(i.eq.1)then
      ninew=ninew+1
      identn(ninew)=ident(j)
      endif
      if(i.gt.1)then
      do 2059 k=1,ninew
      if(ident(j).eq.identn(k))go to 2060
2059  continue
      endif
      write(15,5060)ide(j),formem(j),rd(j),sgmem(j)
5060  format(a11,1x,a15,' R = ',f7.4,2x,I3)
      write(15,'(I3)')mntype(j)
      write(15,'(20I3)')(mns(j,ii),ii=1,mntype(j))
      do kj=1,mntype(j)
      write(15,'(10I4)')(mnr(j,kj,kl),kl=1,10)
      enddo
      ninew=ninew+1
      identn(ninew)=ident(j)
      if(rho(j).lt.fdmin)then
      WRITE(10,2050)RD(J),NT(J),VMEM(J),RHO(J),amem(j),bmem(j),cmem(j),
     1alpmem(j),betmem(j),gammem(j),nmcmem(j),nmem(j),namemem(j),
     2ident(j)
      go to 2060
      endif
      if(rho(j).gt.fdmax)then
      WRITE(10,2051)RD(J),NT(J),VMEM(J),RHO(J),amem(j),bmem(j),cmem(j),
     1alpmem(j),betmem(j),gammem(j),nmcmem(j),nmem(j),namemem(j),
     2ident(j)
      go to 2060
      endif
      WRITE(10,2002)RD(J),NT(J),VMEM(J),RHO(J),amem(j),bmem(j),cmem(j),
     1alpmem(j),betmem(j),gammem(j),nmcmem(j),nmem(j),namemem(j),
     2ident(j)
2060  CONTINUE
C
C
6000  CONTINUE
C
C
C
      if(igood.eq.0)type *,' NO SOLUTION...'
      CALL DATN(DATENOW,TIMENOW)
      CALL CPU_TIME(time_end)
      totaltime=time_end-time_begin
      write(10,*)' Total CPU time elapsed in seconds : ',totaltime
      write(10,*)
      write(10,*)' Total number of runs     : ',iiit
      write(10,*)' Number of structure candidates  : ',iattempt
      write(10,*)' Number of them with Rdt0 < ',rdt0m5,' is : ',iatt5
      write(10,*)' Number of them with Rdt0 < ',rdt0m4,' is : ',iatt4
      write(10,*)' Number of them with Rdt0 < ',rdt0m3,' is : ',iatt3
      write(10,*)' Number of them with Rdt0 < ',rdt0m2,' is : ',iatt2
      write(10,*)' Number of them with Rdt0 < ',rdt0m,' is : ',iatt1
      write(10,*)' Successful retained optimizations : ',igood
      ninew=ninew-1
      if(ninew.eq.-1)ninew=0
      write(10,*)' Unique proposals         : ',ninew
      close(10)
      close(15)
      stop
      end
*-----------------------------------------------------------------------
      subroutine init(iseed)
C
C PORLIB is compiler spcecific part, introduced for 
C using the intrinsic function SECNDS(X) which returns the 
C  (time in seconds since midnight - X) 
C
      USE PORTLIB
      X=0.
      ISEED=INT(GSECNDS(X))*2+1
      RETURN
      END
*
*----------------------------------------------------------------------- 
      function gsecnds(sek)
! machine dependent SECNDS
! replace this line by your specific call
C
C PORLIB is compiler spcecific part, introduced for 
C using the intrinsic function SECNDS(X) which returns the 
C  (time in seconds since midnight - X) 
C
      USE PORTLIB
      gsecnds=SECNDS(sek)
      return
      end
*-----------------------------------------------------------------------
*
*     Subroutine open_read1 opens a file for reading, filling in the
*     supplied extension if none is supplied with the file name.
*     Allows use of system specific facilities of open statement.
*
*
*     file formatted
*
      subroutine open_read1(unit,file)
      integer unit
      character*(*) file
      open (unit,file=file,status='old')
      return
      end
*
*     Subroutine open_write1 opens a file for writing, filling in the
*     supplied extension if none is supplied with the file name.
*     Allows use of system specific facilities of open statement.
*
*     file formatted
*
      subroutine open_write1(unit,file)
      integer unit
      character*(*) file
      open (unit,file=file,status='new')
      return
      end
*
      subroutine filedel(unit,file)
      integer unit
      character*(*) file
      open (unit,file=file,status='old')
      close(unit,status='delete')
      return
      end
*
*-----------------------------------------------------------------------
      SUBROUTINE DATN(DATENOW,TIMENOW)
C
C Obviously, this subroutine is compiler specific...
C
C In case of problem, remove that subroutine and all 
C the CALL DATN inside the program.
C
C Date will be printed as follows :
C
C     10-Apr-2000   Hour:  0 Min: 29 Sec: 20 
C
C     dd-mmm-yyyy          hh     mm      ss 
C
C     DT_VALUES(3),MONTHS(DT_VALUES(2)),DT_VALUES(1), 
C     DT_VALUES(5),DT_VALUES(6),DT_VALUES(7) 
C
C
      CHARACTER*7 DATENOW
      CHARACTER*8 TIMENOW
      CHARACTER*8 DA
      CHARACTER*10 TIME
      CHARACTER*5 ZONE
	CHARACTER*2 YEAR
      INTEGER*4 DT_VALUES(8)    ! Return values from DATE_AND_TIME
      CHARACTER*3 MONTHS(12)    ! Month names
      DATA MONTHS /'Jan','Feb','Mar','Apr','May','Jun',
     1             'Jul','Aug','Sep','Oct','Nov','Dec'/

C
C Get current date values
C
       CALL DATE_AND_TIME (DA,TIME,ZONE,DT_VALUES) 
C
C Format date
C
      WRITE(10,*)
      WRITE(10,10) DT_VALUES(3),
     1 MONTHS(DT_VALUES(2)),DT_VALUES(1),DT_VALUES(5),
     2 DT_VALUES(6),DT_VALUES(7)
      WRITE(10,*)
10    FORMAT(1X,I2,'-',A3,'-',I4,5X,I2,' hour ',I2,' min ',I2,
     -' Sec ')
	IF(DT_VALUES(1).EQ.2002)YEAR='02'
	IF(DT_VALUES(1).EQ.2003)YEAR='03'
	IF(DT_VALUES(1).EQ.2004)YEAR='04'
	IF(DT_VALUES(1).EQ.2005)YEAR='05'
	IF(DT_VALUES(1).EQ.2006)YEAR='06'
	IF(DT_VALUES(1).EQ.2007)YEAR='07'
	IF(DT_VALUES(1).EQ.2008)YEAR='08'
	IF(DT_VALUES(1).EQ.2009)YEAR='09'
	IF(DT_VALUES(1).EQ.2010)YEAR='10'
	IF(DT_VALUES(1).EQ.2011)YEAR='11'
	IF(DT_VALUES(1).EQ.2012)YEAR='12'
	IF(DT_VALUES(1).EQ.2013)YEAR='13'
	IF(DT_VALUES(1).EQ.2014)YEAR='14'
	IF(DT_VALUES(1).EQ.2015)YEAR='15'
	WRITE(DATENOW,'(I2,A3,A2)')
     1DT_VALUES(3),MONTHS(DT_VALUES(2)),YEAR
	WRITE(TIMENOW,'(I2,A,I2,A,I2)')
     1DT_VALUES(5),':',DT_VALUES(6),':',DT_VALUES(7)
C
C  End of compiler specific subroutine
C
      RETURN
      END
*
********************************************************* 
*
      SUBROUTINE SORT(N,A,L)
C     ********************************************************
C     THE SUBROUTINE SORT APPLIES TO THE ARRAY A WITH J ELEMENTS.
C     THE INDICES OF THE ORDERED ARRAY ARE PLACED IN ARRAY L
C     TO OBTAIN THE ORDERED ARRAY REPLACE THE INDICES I OF A(I)
C     WITH I=L(I)---REF.CACM 271
C     *********************************************************
C
      PARAMETER(NGOOD=2000)
      DIMENSION IUT(10),ILT(10)
      DIMENSION A(NGOOD),L(NGOOD)
      J=N
      I=1
      M=1
      DO 10 K=1,J
10    L(K)=K
20    IF(J-I-1)140,140,30
30    IP=(J+I)/2
      ITT=L(IP)
      T=A(ITT)
      L(IP)=L(I)
      IQ=J
      K=I+1
40    IF(K.GT.IQ)GOTO 90
      LK=L(K)
      IF(A(LK).LE.T)GOTO 80
      IQ=IQ
50    IF(IQ.LT.K)GOTO 70
      LQ=L(IQ)
      IF(A(LQ).GE.T)GOTO 60
      IX=L(K)
      L(K)=L(IQ)
      L(IQ)=IX
      IQ=IQ-1
      GOTO 80
60    IQ=IQ-1
      GOTO 50
70    CONTINUE
      IQ=K-1
      GOTO 100
80    K=K+1
      GOTO 40
90    CONTINUE
100   L(I)=L(IQ)
      L(IQ)=ITT
      IF(2*IQ-I-J)120,120,110
110   ILT(M)=I
      IUT(M)=IQ-1
      I=IQ+1
      GOTO 130
120   ILT(M)=IQ+1
      IUT(M)=J
      J=IQ-1
130   M=M+1
      GOTO 20
140   IF(I-J)150,170,170
150   LI=L(I)
      LJ=L(J)
      IF(A(LI)-A(LJ))170,170,160
160   IX=L(I)
      L(I)=L(J)
      L(J)=IX
170   M=M-1
      IF(M)190,190,180
180   I=ILT(M)
      J=IUT(M)
      GOTO 20
190   CONTINUE
      RETURN
      END
*
*
*
********************************************************* 
*
*  Check for environment
*
*
      subroutine complet(nl,x,met,ff,gg,nv,mv,mv2tot,ncop,mcop,ibad)
      parameter (maxa=5184,maxb=139968,nmax=2,maxc=6)
      real     x(maxa,nmax,3),met(3,3),z(maxb,3),ff(2,3),gg(2,3)
      integer  nv(maxa,nmax,maxc),mv(maxa,nmax)
      integer  ncop(2),mcop(maxa)
      common/dist/ y(maxb,3),ny(maxb)
C$OMP THREADPRIVATE(/dist/) 
      ibad=0
      do 4501 ik1=1,nl
      mv(ik1,1)=0
      do 4502 k=1,3
      z(ik1,k)=x(ik1,1,k)
4502  continue
      do 4501 ik3=1,maxc
      nv(ik1,1,ik3)=0
4501  continue
      mv2tot=0
      ni=nl*27
C
C  Expansion to 27 cells
C
      call expan(nl,z,1)
C
C
      do 8000 i=1,nl
      p=x(i,1,1)
      q=x(i,1,2)
      r=x(i,1,3)
      do 8001 j=1,ni
      if(j.le.nl)then
      if(i.eq.ny(j))go to 8001
      endif
      p1=y(j,1)-p
      q1=y(j,2)-q
      r1=y(j,3)-r
      kt=mcop(i)
      if(kt.ne.mcop(ny(j)))kt=3
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.lt.ff(1,kt))then
          ibad=1
          return
        endif
        if(rr.gt.gg(1,kt)) go to 8001
C
C  The good candidate is here ?
C
        mv(i,1)=mv(i,1)+1
        mv2tot=mv2tot+1
        if(mv(i,1).gt.ncop(mcop(i)))then
          ibad=1
          return
        endif
        nv(i,1,mv(i,1))=j
       if(j.gt.5184)type *,' j > 5184 !! ',j
8001  continue
8000  continue
      return
      end
*
*  Adds atoms at bond centres
*
      subroutine midpt(x,nv,mv,nl,ncop,mcop)
*
      parameter (maxa=5184,maxb=139968,nmax=2,maxc=6)
      real     x(maxa,nmax,3)
      integer  nv(maxa,nmax,maxc),mv(maxa,nmax)
      integer  nl(nmax),ncop(2),mcop(maxa)
      common/dist/ y(maxb,3),ny(maxb)
      common/xy/ xy(maxa,3,maxc),nxy(maxa,maxc)
C$OMP THREADPRIVATE(/dist/,/xy/) 
*
*     Find new atom positions
*
      nl1=nl(1)
      nl2=0
      ii=0
      do i=1,nl1
         do j=1,ncop(mcop(i))
            l=nv(i,1,j)
C added OpenMP
            if(l.eq.0) go to 10
C
            if(ny(l).lt.i)go to 10
C
C  Special case if T-T is for the same atom in neighbouring
C            cells : there will be 2 of them, keep only one
C 
            if(ny(l).eq.i)ii=ii+1
            if(ii.eq.2)then
            ii=0
            go to 10
            endif
C
            nl2=nl2+1
            do k=1,3
               dx=(x(i,1,k)+y(l,k))/2.
               xy(i,k,j)=dx
               x(nl2,2,k)=dx
               nv(nl2,2,1)=i
               nv(nl2,2,2)=ny(l)
               mv(nl2,2)=2
            enddo
10         continue
         enddo
      enddo
C
C Coordinates of all T-X neighbours
C
      do i=1,nl1
         do j=1,ncop(mcop(i))
            l=nv(i,1,j)
C      write(10,*)
            do k=1,3
               dx=(x(i,1,k)+y(l,k))/2.
               xy(i,k,j)=dx
            enddo
               nxy(i,j)=0
                  a1=xy(i,1,j)-int(xy(i,1,j))
                  a4=xy(i,2,j)-int(xy(i,2,j))
                  a3=xy(i,3,j)-int(xy(i,3,j))
       if(a1.gt.1.)a1=a1-1.
       if(a1.lt.0.)a1=a1+1.
       if(a4.gt.1.)a4=a4-1.
       if(a4.lt.0.)a4=a4+1.
       if(a3.gt.1.)a3=a3-1.
       if(a3.lt.0.)a3=a3+1.
                 do ij=1,nl2
                  b1=x(ij,2,1)-int(x(ij,2,1))
                  b4=x(ij,2,2)-int(x(ij,2,2))
                  b3=x(ij,2,3)-int(x(ij,2,3))
       if(b1.gt.1.)b1=b1-1.
       if(b1.lt.0.)b1=b1+1.
       if(b4.gt.1.)b4=b4-1.
       if(b4.lt.0.)b4=b4+1.
       if(b3.gt.1.)b3=b3-1.
       if(b3.lt.0.)b3=b3+1.
       ab1=abs(a1-b1)
       ab2=abs(a4-b4)
       ab3=abs(a3-b3)
      if(ab1.lt.0.001.and.ab2.lt.0.001.and.ab3.lt.0.001)then
        nxy(i,j)=ij
C        write(10,*)a1,a4,a3,b1,b4,b3,nxy(i,j)
      endif
                 enddo
         enddo
      enddo
C
C
C
      do i=1,nl1
      mv(i,1)=0
        do j=1,ncop(mcop(i))
        nv(i,1,j)=0
        enddo
      enddo
      do i=1,nl2
      j1=nv(i,2,1)
      mv(j1,1)=mv(j1,1)+1
      nv(j1,1,mv(j1,1))=i
      j2=nv(i,2,2)
      mv(j2,1)=mv(j2,1)+1
      nv(j2,1,mv(j2,1))=i
	enddo
C
      nl(1)=nl1
      nl(2)=nl2
      return
      end
*
*
*
      real function randi(ix)
      implicit integer*4 (a-z)
      real*8 x,s
*
*     A random number generator using the recursion IX=IX*A MOD P
*     where A=7**5 and P=2**31-1. The value returned is in the 
*     range 0.<= ran <1.
*
*     In this form fairly portable as does not require knowledge
*     of data storage, but does not adhere to the standard in two respects:
*     1) Assumes an integer word length of at least 32 bits 
*     2) Assumes that a positive integer less than 2**16 may be
*        floated without loss of digits.
*
*     This code is based on code published by Linus Schrage in
*     T.O.M.S. vol.5 no.2 June 1979 (pp 132-138)
*
*     The method employed is a multiplicative congruential one using a 
*     multiplier of 7**5 and taking the modulo to 2**31-1, i.e. the
*     generator number, x, is updated on each call to the value
*     x*7**5  modulo (2**31-1). The result returned is calculated as a 
*     real number having the value x/(2**31-1)  
*
      parameter (a=16807)
      parameter (b15=32768)
      parameter (b16=65536)
      parameter (p=2147483647)
      parameter (s=1d0/p)
*     a=7**5, b15=2**15, b16=2**16, p= 2**31-1
*
*     Get the 15 high order bits and 16 low order bits of ix
*
      xhi=ix/b16
      xlo=ix-xhi*b16
*
*     Multiply low order part
*
      xalo=xlo*a
*
*     Get high order part of product to carry
*
      leftlo=xalo/b16
*
*     And so obtain high order part of product
*
      xahi=xhi*a+leftlo
*
*     Obtain 32nd bit (overflow) of full product
*
      k=xahi/b15
*
*     Put all the bits of the product together and subtract p
*     (Must be in this form to prevent overflow. The ()'s are essential)
*
      ix=(((xalo-leftlo*b16 )-p) +(xahi-k*b15)*b16) +k
*
*     If <0 add p back again
*
      if (ix.lt.0) ix=ix+p
*
*     Finally multiply by 1/(2**31-1) to obtain number in range 0-1
*
      xhi=ix/b16
      x=dble(xhi)*65536.0d0+dble(ix-xhi*b16)
      x=x*s
      randi=sngl(x)
      return
      end
C
      SUBROUTINE KILL(PRESSEDK)
C
C    Checks if the 'K' keystroke has been pressed
C
      USE DFLIB
      LOGICAL(4) PRESSED / .FALSE. /
      LOGICAL(4) PRESSEDK
      CHARACTER(1) KEY
      PRESSED = PEEKCHARQQ ( )
      IF(PRESSED)THEN
      KEY = GETCHARQQ()
      IF(KEY.EQ.'K')PRESSEDK=.TRUE.
      ENDIF
      RETURN
      END
C
*
*********************************************************************** 
*
      SUBROUTINE DCELL(CELLN,AL,V)
      DIMENSION CELLN(6),AL(3,3),RCELLN(6),CELL(6)
      DEGRAD=3.14159/180.0
      DO 30 I=1,6
30    CELL(I)=CELLN(I)
      DO 31 I=1,3
      L=I+3
      IF(CELL(L)-90.0)32,33,32
33    CELL(L)=0.0
      GOTO 31
32    CELL(L)=COS(DEGRAD*CELL(L))
31    CONTINUE
      CALL TRCL(CELL,RCELLN,V)
C     RCELL IS THE RECIPROCAL CELL CONSTANTS
      DO 34 I=1,3
      AL(I,I)=RCELLN(I)*RCELLN(I)
      CALL PERM(I,J,K)
      IF(J-K)35,36,36
35    AL(J,K)=2.0*RCELLN(J)*RCELLN(K)*RCELLN(I+3)
      GOTO 34
36    AL(K,J)=2.0*RCELLN(J)*RCELLN(K)*RCELLN(I+3) 
34    CONTINUE
C      DO 37 I=4,6
C37    RCELLN(I)=ACOS(RCELLN(I))/DEGRAD
      RETURN
      END
*
*----------------------------------------------------------------------- 
*
      SUBROUTINE TRCL(CELLN,RCELLN,V)
C     TRANSFORMS REAL CELL TO RECIPROCAL OR VICE VERSA 
C     INPUT CELL IS IN ARRAY CELL AS LENGTHS AND COSINES
      DIMENSION CELLN(6),RCELLN(6)
      DIMENSION SINA(3)
      ABC=1.0
      PROD=2.0
      V=-2.0
      DO 10 I=1,3
      L=I+3
      SINA(I)=1.0-CELLN(L)**2
      V=V+SINA(I)
      SINA(I)=SQRT(SINA(I))
      PROD=PROD*CELLN(L)
10    ABC=ABC*CELLN(I)
      V=ABC*SQRT(V+PROD)
C      V IS CELL VOLUME
C     PUT INVERTED CELL INTO RCELL
      DO 20 I=1,3
      CALL PERM(I,J,K)
      RCELLN(I)=CELLN(J)*CELLN(K)*SINA(I)/V
      L=I+3
20    RCELLN(L)=(CELLN(J+3)*CELLN(K+3)-CELLN(L))/(SINA(J)
     2 *SINA(K))
      RETURN
      END
*
*----------------------------------------------------------------------- 
*
      SUBROUTINE PERM(I,J,K)
C     PERMS USEFUL COMBINATIONS OF INTEGERS IN THE RANGE 1 TO 3
      IF(I-2)10,20,30
10    J=2
      K=3
      RETURN
20    J=3
      K=1
      RETURN
30    J=1
      K=2
      RETURN
      END
*
*
********************************************************* 
*
*  Calculate coordination sequences
*
*
      subroutine coorseq(nnl,x,met,gg,ntype,nr,ns,jmax,ncop,mcop,npol,
     1ibad,ncs)
      parameter (maxa=5184,maxb=139968,nmax=2,maxc=6,node=20)
      real         x(maxa,nmax,3),met(3,3),z(maxb,3),gg(2,3)
	integer      nvy(maxb,maxc),mvy(maxb),ncount(maxb),nsi(maxa)
      integer      nnl(2),ncop(2),mcop(maxa)
	integer      ncs(maxa,10),nshelv(10,1000),nr(node,10),ns(node)
      common/dist/ y(maxb,3),ny(maxb)
C$OMP THREADPRIVATE(/dist/) 
      nl=nnl(1)
      do 4001 i=1,nl
      do 4001 k=1,3
      z(i,k)=x(i,1,k)
4001  continue
C
C  First expansion to 27 cells
C
      call expan(nl,z,1)
C
      ni=nl*27
C
C  Reduce coordinates before a second expansion
C      
      do 4002 i=1,ni
      do 4002 k=1,3
      z(i,k)=(y(i,k)+1.)/3.
4002  continue
C
C  Second expansion to 27 cells - total 729 cells
C
      call expan(ni,z,2)
C
C  Rescale y
C
      ni=ni*27
      do 4003 i=1,ni
      do 4003 k=1,3
      y(i,k)=(y(i,k)+1.)/3.
      y(i,k)=y(i,k)*9.-4.5
4003  continue
C
      do 4501 i=1,ni
      mvy(i)=0
      do 4501 j=1,maxc
      nvy(i,j)=0
4501  continue
C   
C
      do 8000 i=1,ni
      p=y(i,1)
      q=y(i,2)
      r=y(i,3)
      k1=ny(i)
      do 8001 j=1,ni
      kt=mcop(k1)
      k2=ny(j)
      kd=mcop(k2)
      if(kt.ne.kd)kt=3
C      if(k1.eq.k2)go to 8001
      p1=y(j,1)-p
      q1=y(j,2)-q
      r1=y(j,3)-r
      rr=met(1,1)*p1*p1+met(2,2)*q1*q1+met(3,3)*r1*r1
     1+met(1,2)*p1*q1+met(1,3)*p1*r1+met(2,3)*q1*r1
        if(rr.gt.gg(1,kt)) go to 8001
        if(rr.lt.0.01) go to 8001
C
C  The good candidate is here ?
C
        mvy(i)=mvy(i)+1
C
C  Care again that the model is OK
C      (not too much first neigbours...) 
C
        if(mvy(i).gt.ncop(mcop(k1)))then
        ibad=1
        return
	  endif
C
        nvy(i,mvy(i))=j
8001  continue
8000  continue
C
C  Calculate the Coordination sequence up to rank 10 for each T atom
C
      jcs=10
      jmax=10
      jmin=10
C
C  Big loop on different T atoms
C
      do 1000 i=1,nl
C
C  10 sequences ?
C
      do 901 j=1,jcs
      ncs(i,j)=0
      do 901 k=1,1000
901   nshelv(j,k)=0
      do 1200 j=1,ni
1200  ncount(j)=1
C
C  shell 1
C
      ncs(i,1)=ncop(mcop(i))
      ncount(i)=0
C
C  prepare shell 2
C
      do 902 k=1,ncs(i,1)
C  order of the atoms in the first shell
      nshelv(1,k)=nvy(i,k)
      ncount(nvy(i,k))=0
902   continue
C
C  manage with shells 2 to 10
C
      do 905 j=2,10
      do 904 kn=1,ncs(i,j-1)
      ik=nshelv(j-1,kn)
       if(ik.eq.0.or.kn.gt.1000)then
       print *,'ik=0 or kn > 1000!',ik,kn
          if(ik.eq.0)then
          ibad=1
          return
          endif
       endif
        if(mvy(ik).lt.ncop(mcop(ny(ik))))then
        jmin=j-1
        if(jmin.lt.jmax)jmax=jmin
        go to 1000
        endif
C
	do 904 k=1,mvy(ik)
      if(ncount(nvy(ik,k)).eq.0)go to 904
      ncs(i,j)=ncs(i,j)+1
      ncount(nvy(ik,k))=0
      nshelv(j,ncs(i,j))=nvy(ik,k)
904   continue
905   continue
C
C
1000  continue
C
C  If some CS at the end are smaller than the previous ones
C        reduce again jmax...
C
      do 1599 i=1,nl
      do 1599 j=2,jmax
      k=jmax-j+1
      ki=k+1
      if(ncs(i,ki).lt.ncs(i,k))then
      jmax=jmax-1
      go to 1598
      endif
1599  continue
C
1598  if(jmax.lt.jcs)then
      do 1600 i=1,nl
      do 1600 j=jmax+1,jcs
1600  ncs(i,j)=0
      endif
C
C      write(10,*)
C      write(10,*)' Coordination sequences'
C      write(10,1501)nl
      do 1500 i=1,nl
      nsi(i)=0
C      write(10,1501)(ncs(i,j),j=1,jmax)
1500  continue
C1501  format(11I4)
C
C  reduce if several identical T sites (with same CS)
C
      ij=1
      ntype=1
1504  iold=ij
      inew=iold
      do 1502 j=1,10
      nr(ntype,j)=ncs(ij,j)
1502  continue
      ns(ntype)=1
      k=ntype
      ik=0
      do 1510 i=ij+1,nl
      if(nsi(i).eq.1)go to 1510
      if(nr(k,1).eq.ncs(i,1).and.nr(k,2).eq.ncs(i,2).and.
     1nr(k,3).eq.ncs(i,3).and.nr(k,4).eq.ncs(i,4).and.
     2nr(k,5).eq.ncs(i,5).and.nr(k,6).eq.ncs(i,6).and.
     3nr(k,7).eq.ncs(i,7).and.nr(k,8).eq.ncs(i,8).and.
     4nr(k,9).eq.ncs(i,9).and.nr(k,10).eq.ncs(i,10))then
      ns(ntype)=ns(ntype)+1
      nsi(i)=1
      else
      ik=ik+1
      if(ik.eq.1)inew=i
      endif
1510  continue
      if(inew.ne.iold)then
      ij=inew
      ntype=ntype+1
      if(ntype.gt.node)then
      print *,'ntype > node',ntype
      ibad=1
      return
      endif
      go to 1504
      endif
C
C
C      write(10,*)
C      write(10,*)' Reduced coordination sequences'
C      write(10,1501)ntype
C      write(10,1501)(ns(i),i=1,ntype)
C      do 1511 i=1,ntype
C      write(10,1501)(nr(i,j),j=1,jmax)
C1511  continue
C
C
      return
      end
C
C  expansion by 27
C
      subroutine expan(nl,x,nt)
      parameter (maxb=139968)
      real         x(maxb,3)
      common/dist/ y(maxb,3),ny(maxb)
C$OMP THREADPRIVATE(/dist/) 
      do 7000 i=1,nl
      if(nt.eq.1)ny(i)=i
      y(i,1)=x(i,1)
      y(i,2)=x(i,2)
      y(i,3)=x(i,3)
7000  continue
      j=0
      do 7001 i=nl+1,2*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)+1.
      y(i,2)=x(j,2)
      y(i,3)=x(j,3)
7001  continue
      j=0
      do 7004 i=2*nl+1,3*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)-1.
      y(i,2)=x(j,2)
      y(i,3)=x(j,3)
7004  continue
      j=0
      do 7002 i=3*nl+1,4*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)
      y(i,2)=x(j,2)+1.
      y(i,3)=x(j,3)
7002  continue
      j=0
      do 7005 i=4*nl+1,5*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)
      y(i,2)=x(j,2)-1.
      y(i,3)=x(j,3)
7005  continue
      j=0
      do 7003 i=5*nl+1,6*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)
      y(i,2)=x(j,2)
      y(i,3)=x(j,3)+1.
7003  continue
      j=0
      do 7006 i=6*nl+1,7*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)
      y(i,2)=x(j,2)
      y(i,3)=x(j,3)-1.
7006  continue
      j=0
      do 7007 i=7*nl+1,8*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)+1.
      y(i,2)=x(j,2)+1.
      y(i,3)=x(j,3)
7007  continue
      j=0
      do 7010 i=8*nl+1,9*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)-1.
      y(i,2)=x(j,2)-1.
      y(i,3)=x(j,3)
7010  continue
      j=0
      do 7008 i=9*nl+1,10*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)
      y(i,2)=x(j,2)+1.
      y(i,3)=x(j,3)+1.
7008  continue
      j=0
      do 7011 i=10*nl+1,11*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)
      y(i,2)=x(j,2)-1.
      y(i,3)=x(j,3)-1.
7011  continue
      j=0
      do 7009 i=11*nl+1,12*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)+1.
      y(i,2)=x(j,2)
      y(i,3)=x(j,3)+1.
7009  continue
      j=0
      do 7012 i=12*nl+1,13*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)-1.
      y(i,2)=x(j,2)
      y(i,3)=x(j,3)-1.
7012  continue
      j=0
      do 7013 i=13*nl+1,14*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)+1.
      y(i,2)=x(j,2)-1.
      y(i,3)=x(j,3)
7013  continue
      j=0
      do 7016 i=14*nl+1,15*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)-1.
      y(i,2)=x(j,2)+1.
      y(i,3)=x(j,3)
7016  continue
      j=0
      do 7014 i=15*nl+1,16*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)
      y(i,2)=x(j,2)+1.
      y(i,3)=x(j,3)-1.
7014  continue
      j=0
      do 7017 i=16*nl+1,17*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)
      y(i,2)=x(j,2)-1.
      y(i,3)=x(j,3)+1.
7017  continue
      j=0
      do 7015 i=17*nl+1,18*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)+1.
      y(i,2)=x(j,2)
      y(i,3)=x(j,3)-1.
7015  continue
      j=0
      do 7018 i=18*nl+1,19*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)-1.
      y(i,2)=x(j,2)
      y(i,3)=x(j,3)+1.
7018  continue
      j=0
      do 7019 i=19*nl+1,20*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)+1.
      y(i,2)=x(j,2)+1.
      y(i,3)=x(j,3)+1.
7019  continue
      j=0
      do 7020 i=20*nl+1,21*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)-1.
      y(i,2)=x(j,2)-1.
      y(i,3)=x(j,3)-1.
7020  continue
      j=0
      do 7021 i=21*nl+1,22*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)-1.
      y(i,2)=x(j,2)+1.
      y(i,3)=x(j,3)+1.
7021  continue
      j=0
      do 7022 i=22*nl+1,23*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)+1.
      y(i,2)=x(j,2)-1.
      y(i,3)=x(j,3)-1.
7022  continue
      j=0
      do 7023 i=23*nl+1,24*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)+1.
      y(i,2)=x(j,2)+1.
      y(i,3)=x(j,3)-1.
7023  continue
      j=0
      do 7024 i=24*nl+1,25*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)-1.
      y(i,2)=x(j,2)-1.
      y(i,3)=x(j,3)+1.
7024  continue
      j=0
      do 7025 i=25*nl+1,26*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)-1.
      y(i,2)=x(j,2)+1.
      y(i,3)=x(j,3)-1.
7025  continue
      j=0
      do 7026 i=26*nl+1,27*nl
      j=j+1
      ny(i)=ny(j)
      y(i,1)=x(j,1)+1.
      y(i,2)=x(j,2)-1.
      y(i,3)=x(j,3)+1.
7026  continue
C
C
      return
      end
C
C  See if there would not be too much T atoms in that small
C      volume if some general or special positions are considered 
C
      subroutine toomuch(itot,npos,nas,v,fdmax3,np2,np1,pp1)
      integer nas(192)
      np2=1
      do 4 is=1,npos
      fdtest=nas(is)+itot
C      fdtest=nas(is)
      fdtest=fdtest/v*1000.
      if(fdtest.lt.fdmax3)go to 5
4     continue
      np2=0
5     np1=is
      pp1=np1
      return
      end
*
*
      SUBROUTINE BIGC(NAM)
      CHARACTER*4 NAM
      DO 1 I=1,2
      IF(NAM(I:I).EQ.'a')NAM(I:I)='A'
      IF(NAM(I:I).EQ.'b')NAM(I:I)='B'
      IF(NAM(I:I).EQ.'c')NAM(I:I)='C'
      IF(NAM(I:I).EQ.'d')NAM(I:I)='D'
      IF(NAM(I:I).EQ.'e')NAM(I:I)='E'
      IF(NAM(I:I).EQ.'f')NAM(I:I)='F'
      IF(NAM(I:I).EQ.'g')NAM(I:I)='G'
      IF(NAM(I:I).EQ.'h')NAM(I:I)='H'
      IF(NAM(I:I).EQ.'i')NAM(I:I)='I'
      IF(NAM(I:I).EQ.'j')NAM(I:I)='J'
      IF(NAM(I:I).EQ.'k')NAM(I:I)='K'
      IF(NAM(I:I).EQ.'l')NAM(I:I)='L'
      IF(NAM(I:I).EQ.'m')NAM(I:I)='M'
      IF(NAM(I:I).EQ.'n')NAM(I:I)='N'
      IF(NAM(I:I).EQ.'o')NAM(I:I)='O'
      IF(NAM(I:I).EQ.'p')NAM(I:I)='P'
      IF(NAM(I:I).EQ.'q')NAM(I:I)='Q'
      IF(NAM(I:I).EQ.'r')NAM(I:I)='R'
      IF(NAM(I:I).EQ.'s')NAM(I:I)='S'
      IF(NAM(I:I).EQ.'t')NAM(I:I)='T'
      IF(NAM(I:I).EQ.'u')NAM(I:I)='U'
      IF(NAM(I:I).EQ.'v')NAM(I:I)='V'
      IF(NAM(I:I).EQ.'w')NAM(I:I)='W'
      IF(NAM(I:I).EQ.'x')NAM(I:I)='X'
      IF(NAM(I:I).EQ.'y')NAM(I:I)='Y'
      IF(NAM(I:I).EQ.'z')NAM(I:I)='Z'
1     CONTINUE
      RETURN
      END
C
C
C
      INTEGER FUNCTION IGCD (M,N)
C
C-----------------------------------------------------------------------
C   IGCD   WRITTEN BY CHARLES P. REEVE, STATISTICAL ENGINEERING
C          DIVISION, NATIONAL BUREAU OF STANDARDS, GAITHERSBURG,
C          MARYLAND  20899
C
C   FOR: COMPUTING THE GREATEST COMMON DIVISOR OF THE INTEGERS M AND N.
C        IF M=N=0 THEN THE G.C.D. IS DEFINED TO BE ZERO.
C
C   SUBPROGRAMS CALLED: -NONE-
C
C   CURRENT VERSION COMPLETED FEBRUARY 3, 1987
C-----------------------------------------------------------------------
C
      I = IABS(N)
      IGCD = IABS(M)
      IF (IGCD.NE.0) GO TO 10 
      IGCD = I
      RETURN
   10 K = MOD(I,IGCD)
      IF (K.EQ.0) RETURN
      I = IGCD
      IGCD = K
      GO TO 10
      END 
      INTEGER FUNCTION IGCDV (M,N)
C
C-----------------------------------------------------------------------
C   IGCDV   WRITTEN BY CHARLES P. REEVE, STATISTICAL ENGINEERING
C           DIVISION, NATIONAL BUREAU OF STANDARDS, GAITHERSBURG,
C           MARYLAND  20899
C
C   FOR: COMPUTING THE GREATEST COMMON DIVISOR OF THE N INTEGERS IN THE
C        VECTOR M.  IF M(1)=M(2)=...=M(N)=0 THEN THE G.C.D. IS DEFINED
C        TO BE ZERO.
C
C   SUBPROGRAMS CALLED: IGCD
C
C   CURRENT VERSION COMPLETED FEBRUARY 3, 1987
C-----------------------------------------------------------------------
C
      DIMENSION M(*)
      IGCDV = M(1)
      DO 10 I = 1, N
         L = IABS(M(I))
         IGCDV = IGCD(IGCDV,L)
         IF (IGCDV.EQ.1) RETURN
   10 CONTINUE
      RETURN
      END 
C
C
C
C     Subroutine for establishing Z and presenting
C                    a better formula
C
      subroutine form(formula,tchem,ntchem,id)
      character*15 text,formula
      character*20 tchem
      character*2 at1,at2,at3
      character*3 aa1,aa2,aa3
      integer na1,na2,na3,nb1,nb2,nb3,nt1,nt2,nt3,ntchem
        na1=2
        na2=2
        na3=2
        nt1=0
        nt2=0
        nt3=0
      text=formula
C
C  estimate Z and reduce the formula
C
        at1=text(1:2)
        if(at1(2:2).eq.' ')na1=1
        at2=text(6:7)
        if(at2(2:2).eq.' ')na2=1
        if(at2(1:1).eq.' ')na2=0
        at3=text(11:12)
        if(at3(2:2).eq.' ')na3=1
        if(at3(1:1).eq.' ')na3=0
        read(text(3:5),*)nt1
        if(na2.ne.0)read(text(8:10),*)nt2
        if(na3.ne.0)read(text(13:15),*)nt3
C       write(*,*)nt1,nt2,nt3
C
C  find the largest common divisor for nt1 & nt2
C
        
        if(nt2.ne.0)then
        id=igcd(nt1,nt2)
        else
        id=nt1
        endif
C
C  verify the correspondence Z*(nt3/id)=nt3
C         if not, Z=Z/2
C
          if(nt3.ne.0)then
          nn=nt3/id
          nz=id*nn
          nn2=id/2
          nz2=2*nn2
          if(nz2.eq.id)then
          if(nz.ne.nt3)id=id/2
          endif
          endif
C
          if(id.ne.0)then
          nt1=nt1/id
          nt2=nt2/id
          nt3=nt3/id
          endif
C       write(*,*)id,nt1,nt2,nt3
        if(nt1.gt.99)then
        write(aa1,'(i3)')nt1
        nb1=3
        else
        if(nt1.gt.9)then
        write(aa1,'(i2)')nt1
        nb1=2
        else
        write(aa1,'(i1)')nt1
        nb1=1
        endif
        endif
        if(nt2.gt.99)then
        write(aa2,'(i3)')nt2
        nb2=3
        else
        if(nt2.gt.9)then
        write(aa2,'(i2)')nt2
        nb2=2
        else
        write(aa2,'(i1)')nt2
        nb2=1
        endif
        endif
        if(nt3.gt.99)then
        write(aa3,'(i3)')nt3
        nb3=3
        else
        if(nt3.gt.9)then
        write(aa3,'(i2)')nt3
        nb3=2
        else
        write(aa3,'(i1)')nt3
        nb3=1
        endif
        endif
      if(nt3.eq.0)then
      if(nt1.eq.1.and.nt2.eq.1)then
      text(1:12)=at1(1:na1)//at2(1:na2)//'           '
      ntchem=na1+na2+nb2+5
      tchem(1:ntchem)='  "'//at1(1:na1)//' '//at2(1:na2)//'"'
      go to 30
      endif
      if(nt1.eq.1)then
      text(1:12)=at1(1:na1)//at2(1:na2)//aa2(1:nb2)//'           '
      ntchem=na1+na2+nb2+5
      tchem(1:ntchem)='  "'//at1(1:na1)//' '//at2(1:na2)//aa2(1:nb2)
     1//'"'
      go to 30
      endif
      if(nt2.eq.1)then
      text(1:12)=at1(1:na1)//aa1(1:nb1)//at2(1:na2)//'           '
      ntchem=na1+nb1+na2+5
      tchem(1:ntchem)='  "'//at1(1:na1)//aa1(1:nb1)//' '//at2(1:na2)
     1//'"'
      go to 30
      endif
      text(1:12)=at1(1:na1)//aa1(1:nb1)//at2(1:na2)//aa2(1:nb2)
     1//'           '
      ntchem=na1+nb1+na2+nb2+5
      tchem(1:ntchem)='  "'//at1(1:na1)//aa1(1:nb1)//' '//at2(1:na2)
     1//aa2(1:nb2)//'"'
      go to 30
      else
      if(nt1.eq.1.and.nt2.eq.1)then
      text(1:12)=at1(1:na1)//at2(1:na2)//at3(1:na3)//aa3(1:nb3)
     1//'       '
      ntchem=na1+na2+na3+nb3+6
      tchem(1:ntchem)='  "'//at1(1:na1)//' '//at2(1:na2)//' '
     1//at3(1:na3)//aa3(1:nb3)//'"'
      go to 30
      endif
      if(nt1.eq.1)then
      text(1:12)=at1(1:na1)//at2(1:na2)//aa2(1:nb2)
     1//at3(1:na3)//aa3(1:nb3)//'       '
      ntchem=na1+na2+nb2+na3+nb3+6
      tchem(1:ntchem)='  "'//at1(1:na1)//' '//at2(1:na2)//aa2(1:nb2)
     1//' '//at3(1:na3)//aa3(1:nb3)//'"'
      go to 30
      endif
      if(nt2.eq.1)then
      text(1:12)=at1(1:na1)//aa1(1:nb1)//at2(1:na2)
     1//at3(1:na3)//aa3(1:nb3)//'       '
      ntchem=na1+nb1+na2+na3+nb3+6
      tchem(1:ntchem)='  "'//at1(1:na1)//aa1(1:nb1)//' '//at2(1:na2)
     1//' '//at3(1:na3)//aa3(1:nb3)//'"'
      go to 30
      endif
      text(1:12)=at1(1:na1)//aa1(1:nb1)//at2(1:na2)//aa2(1:nb2)
     1//at3(1:na3)//aa3(1:nb3)//'       '
      ntchem=na1+nb1+na2+nb2+na3+nb3+6
      tchem(1:ntchem)='  "'//at1(1:na1)//aa1(1:nb1)//' '//at2(1:na2)
     1//aa2(1:nb2)//' '//at3(1:na3)//aa3(1:nb3)//'"'
      go to 30
      endif
30    formula=text
C
      return
      end
C
C
C
C
C ------------------------------------------------------------
C
      SUBROUTINE MCMNAM(LN,NAM)
C
C Get generic filename (NAME) and its length (LN) from the command
C line. LN set to 0 if the file names are to be defined externally
C
      CHARACTER*1 KS
      CHARACTER*80 NAM,KR
      KR=' '
      CALL GETARG(IARGC(),KR)
      LN=0
      NAM=' '
        DO 2 I=1,80
        KS=KR(I:I)
        IF(KS.EQ.' ')GOTO 2
        LN=LN+1
        NAM(LN:LN)=KS
   2    CONTINUE
      RETURN
      END
C
C ------------------------------------------------------------
C

